import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class image:

    @IonosCoreProxy.process_response
    def get_image(self, image_id):
        """
        Retrieves a single image by ID.

        :param      image_id: The unique ID of the image.
        :type       image_id: ``str``

        """
        return self.get_api_instance(ionoscloud.ImageApi)\
            .images_find_by_id_with_http_info(image_id, response_type='object')

    @IonosCoreProxy.process_response
    def list_images(self, depth=1):
        """
        Retrieves a list of images available in the data center.

        :param      depth: The depth of the response data.
        :type       depth: ``int``
        """
        return self.get_api_instance(ionoscloud.ImageApi)\
            .images_get_with_http_info(depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def delete_image(self, image_id):
        """
        Removes only user created images.

        :param      image_id: The unique ID of the image.
        :type       image_id: ``str``

        """
        return self.get_api_instance(ionoscloud.ImageApi).images_delete_with_http_info(image_id)

    @IonosCoreProxy.process_response
    def update_image(self, image_id, **kwargs):
        """
        Replace all properties of an image.

        """
        data = {}

        for attr, value in kwargs.items():
            data[self._underscore_to_camelcase(attr)] = value

        return self.get_api_instance(ionoscloud.ImageApi)\
            .images_patch_with_http_info(image_id, ionoscloud.models.Image(properties=data),
                                         response_type='object')



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import warnings
import unittest

from helpers import configuration
from helpers.resources import resource

from ionosenterprise.client import IonosEnterpriseService
from ionosenterprise.errors import ICNotFoundError


class TestRequest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        cls.requests = cls.client.list_requests()
        cls.request = cls.requests['items'][0]

    def test_list_requests(self):
        requests = self.client.list_requests()

        self.assertGreater(len(requests), 0)
        self.assertEqual(requests['items'][0]['type'], 'request')

    def test_get_request(self):
        request = self.client.get_request(request_id=self.request['id'], status=False)

        self.assertEqual(request['type'], 'request')
        self.assertEqual(request['id'], self.request['id'])
        self.assertEqual(request['href'], self.request['href'])

    def test_get_request_status(self):
        request = self.client.get_request(request_id=self.request['id'], status=True)

        self.assertEqual(request['type'], 'request-status')
        self.assertEqual(request['id'], self.request['id'] + '/status')
        self.assertEqual(request['href'], self.request['href'] + '/status')

    def test_get_failure(self):
        try:
            self.client.get_request(request_id='00000000-0000-0000-0000-000000000000',
                                    status=False)
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])


if __name__ == '__main__':
    unittest.main()



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class request:
    @IonosCoreProxy.process_response
    def get_request(self, request_id, status=False):
        """
        Retrieves a single request by ID.

        :param      request_id: The unique ID of the request.
        :type       request_id: ``str``

        :param      status: Retreive the full status of the request.
        :type       status: ``bool``

        """

        if status:
            return self.get_api_instance(ionoscloud.RequestApi)\
                .requests_status_get_with_http_info(request_id, response_type='object')

        return self.get_api_instance(ionoscloud.RequestApi)\
            .requests_find_by_id_with_http_info(request_id, response_type='object')

    @IonosCoreProxy.process_response
    def list_requests(self, depth=1):
        """
        Retrieves a list of requests available in the account.

        """

        return self.get_api_instance(ionoscloud.RequestApi)\
            .requests_get_with_http_info(depth=depth, response_type='object')



import inspect
import os
import sys
import unittest


def get_source_files():
    """Return a list of sources files/directories (to check with flake8/pylint)"""
    scripts = []
    modules = ["examples", "ionosenterprise", "tests"]
    py_files = ["setup.py"]

    files = []
    for code_file in scripts + modules + py_files:
        is_script = code_file in scripts
        if not os.path.exists(code_file):  # pragma: no cover
            # The alternative path is needed for Debian's pybuild
            alternative = os.path.join(os.environ.get("OLDPWD", ""), code_file)
            code_file = alternative if os.path.exists(alternative) else code_file
        if is_script:
            with open(code_file, "rb") as script_file:
                shebang = script_file.readline().decode("utf-8")
            if ((sys.version_info[0] == 3 and "python3" in shebang)
                    or ("python" in shebang and "python3" not in shebang)):
                files.append(code_file)
        else:
            files.append(code_file)
    return files


def unittest_verbosity():
    """Return the verbosity setting of the currently running unittest
    program, or None if none is running.
    """
    frame = inspect.currentframe()
    while frame:
        self = frame.f_locals.get("self")
        if isinstance(self, unittest.TestProgram):
            return self.verbosity
        frame = frame.f_back
    return None  # pragma: no cover



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class contract:
    @IonosCoreProxy.process_response
    def list_contracts(self, depth=1):
        """
        Retrieves information about the resource limits
        for a particular contract and the current resource usage.

        """
        return self.get_api_instance(ionoscloud.ContractApi)\
            .contracts_get_with_http_info(depth=depth, response_type='object')



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re


def ask(question, options, default):
    """
    Ask the user a question with a list of allowed answers (like yes or no).

    The user is presented with a question and asked to select an answer from
    the given options list. The default will be returned if the user enters
    nothing. The user is asked to repeat his answer if his answer does not
    match any of the allowed anwsers.

    :param    question: Question to present to the user (without question mark)
    :type     question: ``str``

    :param    options: List of allowed anwsers
    :type     options: ``list``

    :param    default: Default answer (if the user enters no text)
    :type     default: ``str``

    """
    assert default in options

    question += " ({})? ".format("/".join(o.upper() if o == default else o for o in options))
    selected = None
    while selected not in options:
        selected = input(question).strip().lower()
        if selected == "":
            selected = default
        else:
            if selected not in options:
                question = "Please type '{}'{comma} or '{}': ".format(
                    "', '".join(options[:-1]), options[-1],
                    comma=',' if len(options) > 2 else '',
                )
    return selected


def find_item_by_name(list_, namegetter, name):
    """
    Find a item a given list by a matching name.

    The search for the name is done in this relaxing way:

    - exact name match
    - case-insentive name match
    - attribute starts with the name
    - attribute starts with the name (case insensitive)
    - name appears in the attribute
    - name appears in the attribute (case insensitive)

    :param    list_: A list of elements
    :type     list_: ``list``

    :param    namegetter: Function that returns the name for a given
                          element in the list
    :type     namegetter: ``function``

    :param    name: Name to search for
    :type     name: ``str``

    """
    matching_items = [i for i in list_ if namegetter(i) == name]
    if not matching_items:
        prog = re.compile(re.escape(name) + '$', re.IGNORECASE)
        matching_items = [i for i in list_ if prog.match(namegetter(i))]
    if not matching_items:
        prog = re.compile(re.escape(name))
        matching_items = [i for i in list_ if prog.match(namegetter(i))]
    if not matching_items:
        prog = re.compile(re.escape(name), re.IGNORECASE)
        matching_items = [i for i in list_ if prog.match(namegetter(i))]
    if not matching_items:
        prog = re.compile(re.escape(name))
        matching_items = [i for i in list_ if prog.search(namegetter(i))]
    if not matching_items:
        prog = re.compile(re.escape(name), re.IGNORECASE)
        matching_items = [i for i in list_ if prog.search(namegetter(i))]
    return matching_items



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
import warnings

from time import sleep
from six import assertRegex

from helpers.resources import resource
from helpers import configuration

from ionosenterprise.client import Datacenter, LoadBalancer,\
    LAN, NIC, Server, IonosEnterpriseService
from ionosenterprise.errors import ICError, ICNotFoundError


class TestLoadBalancer(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        # Create test datacenter.
        cls.datacenter = cls.client.create_datacenter(
            datacenter=Datacenter(**cls.resource['datacenter']))
        cls.client.wait_for_completion(cls.datacenter)

        # Create test LAN.
        cls.lan = cls.client.create_lan(
            datacenter_id=cls.datacenter['id'],
            lan=LAN(**cls.resource['lan']))
        cls.client.wait_for_completion(cls.lan)

        # Create test server.
        cls.server = cls.client.create_server(
            datacenter_id=cls.datacenter['id'],
            server=Server(**cls.resource['server']))
        cls.client.wait_for_completion(cls.server)

        # Create test NIC1.
        nic1 = NIC(**cls.resource['nic'])
        nic1.lan = cls.lan['id']
        cls.nic1 = cls.client.create_nic(
            datacenter_id=cls.datacenter['id'],
            server_id=cls.server['id'],
            nic=nic1)
        cls.client.wait_for_completion(cls.nic1)

        # Create test NIC2.
        # nic2 = NIC(**cls.resource['nic'])
        # nic2.lan = cls.lan['id']
        # cls.nic2 = cls.client.create_nic(
        #     datacenter_id=cls.datacenter['id'],
        #     server_id=cls.server['id'],
        #     nic=nic2)
        # cls.client.wait_for_completion(cls.nic2)

        # Create test LoadBalancer
        loadbalancer = LoadBalancer(**cls.resource['loadbalancer'])
        loadbalancer.balancednics = [cls.nic1['id']]
        cls.loadbalancer = cls.client.create_loadbalancer(
            datacenter_id=cls.datacenter['id'],
            loadbalancer=loadbalancer
        )

        cls.client.wait_for_completion(cls.loadbalancer)

        # Create test LoadBalancer2
        loadbalancer2 = LoadBalancer(**cls.resource['loadbalancer'])
        loadbalancer2.name = "Python SDK Test 2"
        cls.loadbalancer2 = cls.client.create_loadbalancer(
            datacenter_id=cls.datacenter['id'],
            loadbalancer=loadbalancer2
        )

        cls.client.wait_for_completion(cls.loadbalancer2)

        # Create test LoadBalancer3
        loadbalancer3 = LoadBalancer(**cls.resource['loadbalancer'])
        loadbalancer3.balancednics = [cls.nic1['id']]
        loadbalancer3.name = "Python SDK Test 3"
        cls.loadbalancer3 = cls.client.create_loadbalancer(
            datacenter_id=cls.datacenter['id'],
            loadbalancer=loadbalancer3
        )

        cls.client.wait_for_completion(cls.loadbalancer3)

    @classmethod
    def tearDownClass(cls):
        cls.client.delete_datacenter(datacenter_id=cls.datacenter['id'])

    def test_list_loadbalancers(self):
        loadbalancers = self.client.list_loadbalancers(
            datacenter_id=self.datacenter['id'])

        self.assertGreater(len(loadbalancers), 0)
        self.assertEqual(loadbalancers['items'][0]['type'], 'loadbalancer')

    def test_get_loadbalancer(self):
        loadbalancer = self.client.get_loadbalancer(
            datacenter_id=self.datacenter['id'],
            loadbalancer_id=self.loadbalancer['id'])

        self.assertEqual(loadbalancer['type'], 'loadbalancer')
        self.assertEqual(loadbalancer['id'], self.loadbalancer['id'])
        assertRegex(self, loadbalancer['id'], self.resource['uuid_match'])
        self.assertEqual(loadbalancer['properties']['name'],
                         self.loadbalancer['properties']['name'])
        self.assertEqual(loadbalancer['properties']['dhcp'],
                         self.loadbalancer['properties']['dhcp'])
        self.assertIsNotNone(loadbalancer['properties']['ip'])

    def test_delete_loadbalancer(self):
        loadbalancer = self.client.delete_loadbalancer(
            datacenter_id=self.datacenter['id'],
            loadbalancer_id=self.loadbalancer2['id'])

        self.assertTrue(loadbalancer)
        assertRegex(self, loadbalancer['requestId'], self.resource['uuid_match'])

    def test_update_loadbalancer(self):
        loadbalancer = self.client.update_loadbalancer(
            datacenter_id=self.datacenter['id'],
            loadbalancer_id=self.loadbalancer['id'],
            name=self.resource['loadbalancer']['name']+' - RENAME')

        self.assertEqual(loadbalancer['type'], 'loadbalancer')
        self.assertEqual(loadbalancer['properties']['name'],
                         self.resource['loadbalancer']['name']+' - RENAME')

    def test_create_loadbalancer(self):
        self.assertEqual(self.loadbalancer['type'], 'loadbalancer')
        self.assertIsNotNone(self.loadbalancer['entities']['balancednics'])
        self.assertEqual(self.loadbalancer['properties']['name'],
                         self.resource['loadbalancer']['name'])
        self.assertEqual(self.loadbalancer['properties']['dhcp'],
                         self.resource['loadbalancer']['dhcp'])

    def test_associate_nic(self):
        associated_nic = self.client.add_loadbalanced_nics(
            datacenter_id=self.datacenter['id'],
            loadbalancer_id=self.loadbalancer2['id'],
            nic_id=self.nic1['id'])

        self.client.wait_for_completion(associated_nic)

        self.assertEqual(associated_nic['id'], self.nic1['id'])
        self.assertEqual(associated_nic['properties']['name'],
                         self.nic1['properties']['name'])

    def test_remove_nic(self):
        remove_nic = self.client.remove_loadbalanced_nic(
            datacenter_id=self.datacenter['id'],
            loadbalancer_id=self.loadbalancer3['id'],
            nic_id=self.nic1['id'])
        self.assertTrue(remove_nic)
        sleep(30)

    def test_list_balanced_nics(self):
        balanced_nics = self.client.get_loadbalancer_members(
            datacenter_id=self.datacenter['id'],
            loadbalancer_id=self.loadbalancer['id']
        )
        self.assertGreater(len(balanced_nics['items']), 0)
        self.assertEqual(balanced_nics['items'][0]['id'], self.nic1['id'])
        self.assertEqual(balanced_nics['items'][0]['type'], 'nic')

    def test_get_balanced_nic(self):
        balanced_nic = self.client.get_loadbalanced_nic(
            datacenter_id=self.datacenter['id'],
            loadbalancer_id=self.loadbalancer['id'],
            nic_id=self.nic1['id'])

        self.assertEqual(balanced_nic['id'], self.nic1['id'])
        self.assertEqual(balanced_nic['type'], 'nic')
        self.assertEqual(balanced_nic['properties']['name'], self.nic1['properties']['name'])
        self.assertEqual(balanced_nic['properties']['dhcp'], self.nic1['properties']['dhcp'])
        self.assertIsInstance(balanced_nic['properties']['nat'], bool)
        self.assertIsInstance(balanced_nic['properties']['firewallActive'], bool)
        self.assertGreater(len(balanced_nic['properties']['ips']), 0)
        self.assertIsInstance(balanced_nic['properties']['lan'], int)
        assertRegex(self, balanced_nic['properties']['mac'], self.resource['mac_match'])

    def test_get_failure(self):
        try:
            self.client.get_loadbalancer(
                datacenter_id=self.datacenter['id'],
                loadbalancer_id='00000000-0000-0000-0000-000000000000')
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])

    def test_create_failure(self):
        try:
            self.client.create_loadbalancer(
                datacenter_id=self.datacenter['id'],
                loadbalancer=LoadBalancer())
        except ICError as e:
            self.assertIn(self.resource['missing_attribute_error'] % 'lan',
                          e.content[0]['message'])


if __name__ == '__main__':
    unittest.main()



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import warnings
import uuid
import unittest

from six import assertRegex

from helpers import configuration
from helpers.resources import resource

from ionosenterprise.client import Datacenter, Server, LAN, NIC, \
    IonosEnterpriseService, PrivateCrossConnect
from ionosenterprise.errors import ICNotFoundError


class TestLan(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        # Create test datacenter.
        cls.datacenter = cls.client.create_datacenter(
            datacenter=Datacenter(**cls.resource['datacenter']))
        cls.client.wait_for_completion(cls.datacenter)

        # Create pcc.
        pcc = PrivateCrossConnect(name="TEST NAME - %s" % uuid.uuid1(),
                                  description="TEST DESCRIPTION 1")

        cls.pcc = cls.client.create_pcc(pcc)
        cls.client.wait_for_completion(cls.pcc)

        # Create test LAN.
        lan_properties = cls.resource['lan']
        lan_properties['pcc_id'] = cls.pcc['id']
        cls.lan = cls.client.create_lan(
            datacenter_id=cls.datacenter['id'],
            lan=LAN(**lan_properties))
        cls.client.wait_for_completion(cls.lan)

        # Create test server.
        cls.server = cls.client.create_server(
            datacenter_id=cls.datacenter['id'],
            server=Server(**cls.resource['server']))
        cls.client.wait_for_completion(cls.server)

        # Create test NIC1.
        nic1 = NIC(**cls.resource['nic'])
        nic1.lan = cls.lan['id']
        cls.nic1 = cls.client.create_nic(
            datacenter_id=cls.datacenter['id'],
            server_id=cls.server['id'],
            nic=nic1)
        cls.client.wait_for_completion(cls.nic1)

        # Create test NIC2.
        nic2 = NIC(**cls.resource['nic'])
        nic2.lan = cls.lan['id']
        cls.nic2 = cls.client.create_nic(
            datacenter_id=cls.datacenter['id'],
            server_id=cls.server['id'],
            nic=nic2)
        cls.client.wait_for_completion(cls.nic2)

    @classmethod
    def tearDownClass(cls):
        cls.client.delete_nic(cls.datacenter['id'], cls.server['id'], cls.nic1['id'])
        cls.client.delete_nic(cls.datacenter['id'], cls.server['id'], cls.nic2['id'])
        cls.client.delete_datacenter(datacenter_id=cls.datacenter['id'])

    def test_list_lans(self):
        lans = self.client.list_lans(datacenter_id=self.datacenter['id'])

        self.assertGreater(len(lans), 0)
        self.assertEqual(lans['items'][0]['type'], 'lan')
        self.assertIn(lans['items'][0]['id'], ('1', '2', '3'))
        self.assertEqual(lans['items'][0]['properties']['name'], self.resource['lan']['name'])
        self.assertEqual(lans['items'][0]['properties']['public'], self.resource['lan']['public'])

    def test_get_lan(self):
        lan = self.client.get_lan(datacenter_id=self.datacenter['id'], lan_id=self.lan['id'])

        self.assertEqual(lan['type'], 'lan')
        self.assertEqual(lan['id'], self.lan['id'])
        self.assertEqual(lan['properties']['name'], self.resource['lan']['name'])
        self.assertEqual(lan['properties']['public'], self.resource['lan']['public'])

    def test_remove_lan(self):
        lan = self.client.create_lan(
            datacenter_id=self.datacenter['id'],
            lan=LAN(**self.resource['lan']))

        self.client.wait_for_completion(lan)

        lan = self.client.delete_lan(datacenter_id=self.datacenter['id'], lan_id=lan['id'])

        self.assertTrue(lan)
        assertRegex(self, lan['requestId'], self.resource['uuid_match'])

    def test_update_lan(self):
        lan = self.client.update_lan(
            datacenter_id=self.datacenter['id'],
            lan_id=self.lan['id'],
            name=self.resource['lan']['name'] + ' - RENAME',
            public=False)

        self.assertEqual(lan['type'], 'lan')
        self.assertEqual(lan['properties']['name'], self.resource['lan']['name'] + ' - RENAME')
        self.assertFalse(lan['properties']['public'])

    def test_create_lan(self):
        self.assertEqual(self.lan['id'], '1')
        self.assertEqual(self.lan['type'], 'lan')
        self.assertEqual(self.lan['properties']['name'], self.resource['lan']['name'])
        self.assertEqual(self.lan['properties']['public'], self.resource['lan']['public'])

    def test_create_complex_lan(self):
        nic_resource = NIC(**self.resource['nic'])

        nic1 = self.client.create_nic(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'],
            nic=nic_resource)
        self.client.wait_for_completion(nic1)
        self.assertFalse(nic1['properties']['nat'])
        self.assertEqual(nic1['properties']['name'], 'Python SDK Test')
        self.assertTrue(nic1['properties']['dhcp'])
        self.assertEqual(nic1['properties']['lan'], 1)
        self.assertTrue(nic1['properties']['firewallActive'])

        nics = [nic1['id']]
        lan = LAN(nics=nics, **self.resource['lan'])

        response = self.client.create_lan(
            datacenter_id=self.datacenter['id'],
            lan=lan)
        self.client.wait_for_completion(response)

        self.assertEqual(response['type'], 'lan')
        self.assertEqual(response['properties']['name'], self.resource['lan']['name'])
        self.assertFalse(response['properties']['public'])

    def test_get_lan_members(self):
        members = self.client.get_lan_members(
            datacenter_id=self.datacenter['id'],
            lan_id=self.lan['id'])

        self.assertGreater(len(members), 0)
        self.assertEqual(members['items'][0]['type'], 'nic')
        self.assertEqual(members['items'][0]['properties']['name'], self.resource['nic']['name'])
        assertRegex(self, members['items'][0]['properties']['mac'], self.resource['mac_match'])

    def test_get_failure(self):
        try:
            self.client.get_lan(datacenter_id=self.datacenter['id'], lan_id=0)
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])

    def test_create_failure(self):
        try:
            self.client.create_lan(
                datacenter_id='00000000-0000-0000-0000-000000000000', lan=LAN())
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])


if __name__ == '__main__':
    unittest.main()



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class ipblock:
    @IonosCoreProxy.process_response
    def get_ipblock(self, ipblock_id):
        """
        Retrieves a single IP block by ID.

        :param      ipblock_id: The unique ID of the IP block.
        :type       ipblock_id: ``str``

        """
        return self.get_api_instance(ionoscloud.IPBlocksApi)\
            .ipblocks_find_by_id_with_http_info(ipblock_id, response_type='object')

    @IonosCoreProxy.process_response
    def list_ipblocks(self, depth=1):
        """
        Retrieves a list of IP blocks available in the account.

        """
        return self.get_api_instance(ionoscloud.IPBlocksApi)\
            .ipblocks_get_with_http_info(depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def delete_ipblock(self, ipblock_id):
        """
        Removes a single IP block from your account.

        :param      ipblock_id: The unique ID of the IP block.
        :type       ipblock_id: ``str``

        """
        return self.get_api_instance(ionoscloud.IPBlocksApi)\
            .ipblocks_delete_with_http_info(ipblock_id)

    @IonosCoreProxy.process_response
    def reserve_ipblock(self, ipblock):
        """
        Reserves an IP block within your account.

        """
        properties = {
            "name": ipblock.name
        }

        if ipblock.location:
            properties['location'] = ipblock.location

        if ipblock.size:
            properties['size'] = str(ipblock.size)

        ipblock = ionoscloud.models.IpBlock(
            properties=properties
        )

        return self.get_api_instance(ionoscloud.IPBlocksApi)\
            .ipblocks_post_with_http_info(ipblock, response_type='object')



#!/usr/bin/python
# encoding: utf-8

# Copyright 2016-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=global-variable-undefined

'''
pb_createDatacenter is a sample script to create a complex datacenter.


@author:     Jürgen Buchhammer

@copyright:  2016 IONOS. All rights reserved.

@license:    Apache License 2.0

@deffield    updated: Updated
'''

import sys
import os

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter

from time import sleep
import json
from base64 import b64decode, b64encode

from ionosenterprise.client import IonosEnterpriseService
from ionosenterprise.client import Datacenter, Volume, Server
from ionosenterprise.client import LAN, NIC, FirewallRule


__version__ = 0.2
__date__ = '2016-09-26'
__updated__ = __date__


def getLogin(filename, user, passwd):
    '''
    write user/passwd to login file or get them from file.
    This method is not Py3 safe (byte vs. str)
    '''
    if filename is None:
        return (user, passwd)
    isPy2 = sys.version_info[0] == 2
    if os.path.exists(filename):
        print("Using file {} for Login".format(filename))
        with open(filename, "r") as loginfile:
            encoded_cred = loginfile.read()
            print("encoded: {}".format(encoded_cred))
            if isPy2:
                decoded_cred = b64decode(encoded_cred)
            else:
                decoded_cred = b64decode(encoded_cred).decode('utf-8')
            login = decoded_cred.split(':', 1)
            return (login[0], login[1])
    else:
        if user is None or passwd is None:
            raise ValueError("user and password must not be None")
        print("Writing file {} for Login".format(filename))
        with open(filename, "wb") as loginfile:
            creds = user+":"+passwd
            if isPy2:
                encoded_cred = b64encode(creds)
            else:
                encoded_cred = b64encode(creds.encode('utf-8'))
            print("encoded: {}".format(encoded_cred))
            loginfile.write(encoded_cred)
        return (user, passwd)
# end getLogin()


def wait_for_request(pbclient, request_id,
                     timeout=0, initial_wait=5, scaleup=10):
    '''
    Waits for a request to finish until timeout.
    timeout==0 is interpreted as infinite wait time.
    Returns a tuple (return code, request status, message) where return code
    0  : request successful
    1  : request failed
    -1 : timeout exceeded
    The wait_period is increased every scaleup steps to adjust for long
    running requests.
    '''
    total_wait = 0
    wait_period = initial_wait
    next_scaleup = scaleup * wait_period
    wait = True
    while wait:
        request_status = pbclient.get_request(request_id, status=True)
        state = request_status['metadata']['status']
        if state == "DONE":
            return(0, state, request_status['metadata']['message'])
        if state == 'FAILED':
            return(1, state, request_status['metadata']['message'])
        print("Request '{}' is in state '{}'. Sleeping for {} seconds..."
              .format(request_id, state, wait_period))
        sleep(wait_period)
        total_wait += wait_period
        if timeout != 0 and total_wait > timeout:
            wait = False
        next_scaleup -= wait_period
        if next_scaleup == 0:
            wait_period += initial_wait
            next_scaleup = scaleup * wait_period
            print("scaling up wait_period to {}, next change in {} seconds"
                  .format(wait_period, next_scaleup))
    # end while(wait)
    return (-1, state, "request not finished before timeout")
# end wait_for_request()


def wait_for_requests(pbclient, request_ids=None,
                      timeout=0, initial_wait=5, scaleup=10):
    '''
    Waits for a list of requests to finish until timeout.
    timeout==0 is interpreted as infinite wait time.
    Returns a dict of request_id -> result.
    result is a tuple (return code, request status, message) where return code
    0  : request successful
    1  : request failed
    -1 : timeout exceeded
    The wait_period is increased every scaleup steps to adjust for long
    running requests.
    '''
    done = dict()
    if not request_ids:
        print("empty request list")
        return done
    total_wait = 0
    wait_period = initial_wait
    next_scaleup = scaleup * wait_period
    wait = True
    while wait:
        for request_id in request_ids:
            if request_id in done:
                continue
            request_status = pbclient.get_request(request_id, status=True)
            state = request_status['metadata']['status']
            if state == "DONE":
                done[request_id] = (0, state, request_status['metadata']['message'])
                print("Request '{}' is in state '{}'.".format(request_id, state))
            if state == 'FAILED':
                done[request_id] = (1, state, request_status['metadata']['message'])
                print("Request '{}' is in state '{}'.".format(request_id, state))
        # end for(request_ids)
        if len(done) == len(request_ids):
            wait = False
        else:
            print("{} of {} requests are finished. Sleeping for {} seconds..."
                  .format(len(done), len(request_ids), wait_period))
            sleep(wait_period)
            total_wait += wait_period
            if timeout != 0 and total_wait > timeout:
                wait = False
            next_scaleup -= wait_period
            if next_scaleup == 0:
                wait_period += initial_wait
                next_scaleup = scaleup * wait_period
                print("scaling up wait_period to {}, next change in {} seconds"
                      .format(wait_period, next_scaleup))
        # end if/else(done)
    # end while(wait)
    if len(done) != len(request_ids):
        for request_id in request_ids:
            if request_id in done:
                continue
            done[request_id] = (-1, state, "request not finished before timeout")
    return done
# end wait_for_requests()


def create_datacenter_dict(pbclient, datacenter):
    """
    Creates a Datacenter dict -- both simple and complex are supported.
    This is copied from createDatacenter() and uses private methods.

    """
    # pylint: disable=protected-access
    server_items = []
    volume_items = []
    lan_items = []
    loadbalancer_items = []

    entities = dict()

    properties = {
        "name": datacenter.name,
        "location": datacenter.location,
    }
    # Optional Properties
    if datacenter.description:
        properties['description'] = datacenter.description

    # Servers
    if datacenter.servers:
        for server in datacenter.servers:
            server_items.append(pbclient._create_server_dict(server))
        servers = {
            "items": server_items
        }
        server_entities = {
            "servers": servers
        }
        entities.update(server_entities)

    # Volumes
    if datacenter.volumes:
        for volume in datacenter.volumes:
            volume_items.append(pbclient._create_volume_dict(volume))
        volumes = {
            "items": volume_items
        }
        volume_entities = {
            "volumes": volumes
        }
        entities.update(volume_entities)

    # Load Balancers
    if datacenter.loadbalancers:
        for loadbalancer in datacenter.loadbalancers:
            loadbalancer_items.append(
                pbclient._create_loadbalancer_dict(
                    loadbalancer
                    )
                )
        loadbalancers = {
            "items": loadbalancer_items
        }
        loadbalancer_entities = {
            "loadbalancers": loadbalancers
        }
        entities.update(loadbalancer_entities)

    # LANs
    if datacenter.lans:
        for lan in datacenter.lans:
            lan_items.append(
                pbclient._create_lan_dict(lan)
                )
        lans = {
            "items": lan_items
        }
        lan_entities = {
            "lans": lans
        }
        entities.update(lan_entities)

    if not entities:
        raw = {
            "properties": properties,
        }
    else:
        raw = {
            "properties": properties,
            "entities": entities
        }

    return raw
# end create_datacenter_dict()


def write_dc_definition(dcdef=None, filename=None):
    with open(filename, 'w') as outfile:
        json.dump(dcdef, outfile, indent=2)
    return 0
# end write_dc_definition()


def read_dc_definition(filename=None):
    with open(filename) as infile:
        dcdef = json.load(infile)
    return dcdef
# end read_dc_definition()


# -- build objects from internal dict structures --

def getDatacenterObject(defdict=None):
    if not defdict or not isinstance(defdict, dict):
        raise ValueError("argument 'defdict' must be non-empty dict")
    props = dict()
    for k, v in defdict['properties'].items():
        # no renaming needed here, but this accounts for optional props too
        props[k] = v
    apiobj = Datacenter(**props)
    return apiobj
# end getDatacenterObject()


def getVolumeObject(defdict=None):
    if not defdict or not isinstance(defdict, dict):
        raise ValueError("argument 'defdict' must be non-empty dict")
    # TODO: can we set deviceNumber too? Nope, not supported in Volume()
    # AARGH! some of Volume's fields have different names -> need to convert
    # so make a copy and let source as is
    props = dict()
    for k, v in defdict['properties'].items():
        if k == 'type':
            props['disk_type'] = v
            continue
        if k == 'imagePassword':
            props['image_password'] = v
            continue
        if k == 'licenceType':
            props['licence_type'] = v
            continue
        if k == 'sshKeys':
            props['ssh_keys'] = v
            continue
        props[k] = v
    # end for(defdict)
    apiobj = Volume(**props)
    return apiobj
# end getVolumeObject()


def getServerObject(defdict=None):
    if not defdict or not isinstance(defdict, dict):
        raise ValueError("argument 'defdict' must be non-empty dict")
    # AARGH! some of Servers's fields have different names -> need to convert manually
    # so make a copy and let source as is
    props = dict()
    for k, v in defdict['properties'].items():
        if k == 'availabilityZone':
            props['availability_zone'] = v
            continue
        if k == 'bootCdrom':
            props['boot_cdrom'] = v
            continue
        if k == 'cpuFamily':
            props['cpu_family'] = v
            continue
# -- TODO: if volumes entries have an ID -> attach, else create w/ properties of volume
#         if k == 'bootVolume':
#             props['boot_volume_id'] = v
#             continue
#         if k == 'volumes':
#             props['create_volumes'] = v
#             continue
#         if k == 'volumes':
#             props['attach_volumes'] = v
#             continue
# Server() has no kwargs, so we must exactly match the known keywords
        if k in ['name', 'cores', 'ram']:
            props[k] = v
    # end for(defdict)
    apiobj = Server(**props)
    return apiobj
# end getServerObject()


def getNICObject(defdict=None):
    if not defdict or not isinstance(defdict, dict):
        raise ValueError("argument 'defdict' must be non-empty dict")
    # AARGH! some of NIC's fields have different names -> need to convert manually
    # so make a copy and let source as is
    props = dict()
    for k, v in defdict['properties'].items():
        if k == 'firewallActive':
            props['firewall_active'] = v
            continue
        if k == 'firewallrules':
            # this needs more to do:
            # if we have rules we must get rules objects too, irrespective of being active
            if v:
                rules = [getFwRuleObject(rule) for rule in v]
                props['firewall_rules'] = rules
                continue
        props[k] = v
    # end for(defdict)
    apiobj = NIC(**props)
    return apiobj
# end getNICObject()


def getFwRuleObject(defdict=None):
    if not defdict or not isinstance(defdict, dict):
        raise ValueError("argument 'defdict' must be non-empty dict")
    # AARGH! some of NIC's fields have different names -> need to convert manually
    # so make a copy and let source as is
    props = dict()
    for k, v in defdict['properties'].items():
        if k == 'sourceMac':
            props['source_mac'] = v
            continue
        if k == 'sourceIp':
            props['source_ip'] = v
            continue
        if k == 'targetIp':
            props['target_ip'] = v
            continue
        if k == 'portRangeStart':
            props['port_range_start'] = v
            continue
        if k == 'portRangeEnd':
            props['port_range_end'] = v
            continue
        if k == 'icmpType':
            props['icmp_type'] = v
            continue
        if k == 'icmpCode':
            props['icmp_code'] = v
            continue
        props[k] = v
    # end for(defdict)
    apiobj = FirewallRule(**props)
    return apiobj
# end getFwRuleObject()


def getLANObject(defdict=None):
    if not defdict or not isinstance(defdict, dict):
        raise ValueError("argument 'defdict' must be non-empty dict")
    props = dict()
    for k, v in defdict['properties'].items():
        # no renaming needed here, but this accounts for optional props too
        props[k] = v
    # end for(defdict)
    apiobj = LAN(**props)
    return apiobj
# end getLANObject()


# -- the (far too long) main method --

def main(argv=None):
    '''Parse command line options and create a server/volume composite.'''

    if argv is None:
        argv = sys.argv
    else:
        sys.argv.extend(argv)

    program_name = os.path.basename(sys.argv[0])
    program_version = "v%s" % __version__
    program_build_date = str(__updated__)
    program_version_message = '%%(prog)s %s (%s)' % (program_version,
                                                     program_build_date)
    program_shortdesc = __import__('__main__').__doc__.split("\n")[1]
    program_license = '''%s

  Created by J. Buchhammer on %s.
  Copyright 2016 IONOS. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc, str(__date__))
    # Setup argument parser
    parser = ArgumentParser(description=program_license,
                            formatter_class=RawDescriptionHelpFormatter)
    parser.add_argument('-u', '--user', dest='user', help='the login name')
    parser.add_argument('-p', '--password', dest='password',
                        help='the login password')
    parser.add_argument('-L', '--Login', dest='loginfile', default=None,
                        help='the login file to use')
    parser.add_argument('-i', '--infile', dest='infile', default=None,
                        required=True, help='the input file name')
    parser.add_argument('-D', '--DCname', dest='dcname', default=None,
                        help='new datacenter name')
# TODO: add/overwrite image password for creation
#    parser.add_argument('-P', '--imagepassword', dest='imgpassword',
#                        default=None, help='the image password')
    parser.add_argument('-v', '--verbose', dest="verbose", action="count",
                        default=0, help="set verbosity level [default: %(default)s]")
    parser.add_argument('-V', '--version', action='version',
                        version=program_version_message)

    # Process arguments
    args = parser.parse_args()
    global verbose
    verbose = args.verbose

    if verbose > 0:
        print("Verbose mode on")
        print("start {} with args {}".format(program_name, str(args)))

    (user, password) = getLogin(args.loginfile, args.user, args.password)
    if user is None or password is None:
        raise ValueError("user or password resolved to None")
    pbclient = IonosEnterpriseService(user, password)

    usefile = args.infile
    print("read dc from {}".format(usefile))
    dcdef = read_dc_definition(usefile)
    if verbose > 0:
        print("using DC-DEF {}".format(json.dumps(dcdef)))

    # setup dc:
    #     + create empty dc
    #     + create volumes (unattached), map uuid to servers (custom dict)
    #     + create servers
    #     + create nics
    #     + attach volumes

    if 'custom' in dcdef and 'id' in dcdef['custom']:
        dc_id = dcdef['custom']['id']
        print("using existing DC w/ id {}".format(str(dc_id)))
    else:
        if args.dcname is not None:
            print("Overwrite DC name w/ '{}'".format(args.dcname))
            dcdef['properties']['name'] = args.dcname
        dc = getDatacenterObject(dcdef)
        # print("create DC {}".format(str(dc)))
        response = pbclient.create_datacenter(dc)
        dc_id = response['id']
        if 'custom' not in dcdef:
            dcdef['custom'] = dict()
        dcdef['custom']['id'] = dc_id
        result = wait_for_request(pbclient, response['requestId'])
        print("wait loop returned {}".format(result))
        tmpfile = usefile+".tmp_postdc"
        write_dc_definition(dcdef, tmpfile)

    requests = []
    print("create Volumes {}".format(str(dc)))
    # we do NOT consider dangling volumes, only server-attached ones
    for server in dcdef['entities']['servers']['items']:
        print("- server {}".format(server['properties']['name']))
        if 'volumes' not in server['entities']:
            print(" server {} has no volumes".format(server['properties']['name']))
            continue
        for volume in server['entities']['volumes']['items']:
            if 'custom' in volume and 'id' in volume['custom']:
                vol_id = volume['custom']['id']
                print("using existing volume w/ id {}".format(str(vol_id)))
            else:
                dcvol = getVolumeObject(volume)
                print("OBJ: {}".format(str(dcvol)))
                response = pbclient.create_volume(dc_id, dcvol)
                volume.update({'custom': {'id': response['id']}})
                requests.append(response['requestId'])
        # end for(volume)
    # end for(server)
    if requests:
        result = wait_for_requests(pbclient, requests, initial_wait=10, scaleup=15)
        print("wait loop returned {}".format(str(result)))
        tmpfile = usefile+".tmp_postvol"
        write_dc_definition(dcdef, tmpfile)
    else:
        print("all volumes existed already")

    requests = []
    print("create Servers {}".format(str(dc)))
    # we do NOT consider dangling volumes, only server-attached ones
    for server in dcdef['entities']['servers']['items']:
        print("- server {}".format(server['properties']['name']))
        if 'custom' in server and 'id' in server['custom']:
            srv_id = server['custom']['id']
            print("using existing server w/ id {}".format(str(srv_id)))
        else:
            dcsrv = getServerObject(server)
            print("OBJ: {}".format(str(dcsrv)))
            response = pbclient.create_server(dc_id, dcsrv)
            server.update({'custom': {'id': response['id']}})
            requests.append(response['requestId'])
    # end for(server)
    if requests:
        result = wait_for_requests(pbclient, requests, initial_wait=10, scaleup=15)
        print("wait loop returned {}".format(str(result)))
        tmpfile = usefile+".tmp_postsrv"
        write_dc_definition(dcdef, tmpfile)
    else:
        print("all servers existed already")

# TODO: only do this if we have lan entities
    requests = []
    # Huuh, looks like we get unpredictable order for LANs!
    # Nope, order of creation determines the LAN id,
    # thus we better wait for each request
    print("create LANs {}".format(str(dc)))
    for lan in dcdef['entities']['lans']['items']:
        print("- lan {}".format(lan['properties']['name']))
        dclan = getLANObject(lan)
        print("OBJ: {}".format(str(dclan)))
        response = pbclient.create_lan(dc_id, dclan)
        lan.update({'custom': {'id': response['id']}})
        result = wait_for_request(pbclient, response['requestId'])
        print("wait loop returned {}".format(str(result)))
    # end for(lan)
    tmpfile = usefile+".tmp_postlan"
    write_dc_definition(dcdef, tmpfile)

    requests = []
    # Attention:
    # NICs appear in OS in the order, they are created.
    # But DCD rearranges the display by ascending MAC addresses.
    # This does not change the OS order.
    # MAC may not be available from create response,
    # thus we wait for each request :-(
    print("create NICs {}".format(str(dc)))
    for server in dcdef['entities']['servers']['items']:
        print("- server {}".format(server['properties']['name']))
        srv_id = server['custom']['id']
        if 'nics' not in server['entities']:
            print(" server {} has no NICs".format(server['properties']['name']))
            continue
        macmap = dict()
        for nic in server['entities']['nics']['items']:
            dcnic = getNICObject(nic)
            response = pbclient.create_nic(dc_id, srv_id, dcnic)
            # print("dcnic response {}".format(str(response)))
            # mac = response['properties']['mac'] # we don't get it here !?
            nic_id = response['id']
            result = wait_for_request(pbclient, response['requestId'])
            print("wait loop returned {}".format(str(result)))
            response = pbclient.get_nic(dc_id, srv_id, nic_id, 2)
            mac = response['properties']['mac']
            print("dcnic has MAC {} for {}".format(mac, nic_id))
            macmap[mac] = nic_id
        # end for(nic)
        macs = sorted(macmap)
        print("macs will be displayed by DCD in th following order:")
        for mac in macs:
            print("mac {} -> id{}".format(mac, macmap[mac]))
    # end for(server)
    tmpfile = usefile+".tmp_postnic"
    write_dc_definition(dcdef, tmpfile)

    requests = []
    # don't know if we get a race here too, so better wait for each request :-/
    print("attach volumes {}".format(str(dc)))
    for server in dcdef['entities']['servers']['items']:
        print("- server {}".format(server['properties']['name']))
        if 'volumes' not in server['entities']:
            print(" server {} has no volumes".format(server['properties']['name']))
            continue
        srv_id = server['custom']['id']
        for volume in server['entities']['volumes']['items']:
            print("OBJ: {}".format(volume['properties']['name']))
            response = pbclient.attach_volume(dc_id, srv_id, volume['custom']['id'])
            result = wait_for_request(pbclient, response['requestId'])
            print("wait loop returned {}".format(str(result)))
        # end for(volume)
    # end for(server)
    tmpfile = usefile+".tmp_postatt"
    write_dc_definition(dcdef, tmpfile)

    # TODO: do we need to set boot volume for each server?
    # looks like it's working without

    return 0

# end main()


if __name__ == "__main__":
    sys.exit(main())



class LoadBalancer(object):
    """
    This is the main class for managing load balancer resources.
    """

    def __init__(self, name=None, ip=None,  # pylint: disable=unused-argument
                 dhcp=None, balancednics=None):
        """
        LoadBalancer class initializer.

        :param      name: The name of the load balancer.
        :type       name: ``str``

        :param      ip: The IP for the load balancer.
        :type       ip: ``str``

        :param      dhcp: Indicates if the load balancer
                          uses DHCP or not.
        :type       dhcp: ``bool``

        :param      balancednics: A list of NICs associated
                                  with the load balancer.
        :type       balancednics: ``list``

        """
        if balancednics is None:
            balancednics = []
        self.name = name
        self.ip = ip
        self.dhcp = dhcp
        self.balancednics = balancednics

    def __repr__(self):
        return (('<LoadBalancer: name=%s, ip=%s, dhcp=%s> ...>')
                % (self.name, self.ip, str(self.dhcp)))



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
import warnings

from helpers import configuration
from helpers.resources import resource

from ionosenterprise.client import Datacenter, IonosEnterpriseService, Server


class TestCdrom(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings\
            .filterwarnings("ignore", category=ResourceWarning,
                            message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        # Create test datacenter.
        cls.datacenter = cls.client.create_datacenter(Datacenter(**cls.resource['datacenter']))
        cls.client.wait_for_completion(cls.datacenter)

        # Create test server.
        cls.server = cls.client.create_server(cls.datacenter['id'],
                                              Server(**cls.resource['server']))
        cls.client.wait_for_completion(cls.server)

        # Use an image ID for CDROM
        images = cls.client.list_images()['items']
        images = list(filter(lambda v:
                             v['properties']['location'] == cls.resource['datacenter']['location']
                             and v['properties']['imageType'] != 'HDD',
                             images))
        cls.image_id = images[0]['id']
        cls.attached_cdrom = cls.client.attach_cdrom(cls.datacenter['id'],
                                                     cls.server['id'], cls.image_id)
        cls.client.wait_for_completion(cls.attached_cdrom)

    @classmethod
    def tearDownClass(cls):
        cls.client.detach_cdrom(cls.datacenter['id'], cls.server['id'], cls.image_id)
        cls.client.delete_server(cls.datacenter['id'], cls.server['id'])
        cls.client.delete_datacenter(cls.datacenter['id'])

    def test_get_attached_cdroms(self):
        response = self.client.get_attached_cdroms(self.datacenter['id'], self.server['id'])
        self.assertGreater(len(response['items']), 0)

    def test_get_attached_cdrom(self):
        response = self.client.get_attached_cdrom(self.datacenter['id'],
                                                  self.server['id'], self.image_id)
        self.assertEqual(response['id'], self.image_id)



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
import warnings

from six import assertRegex

from helpers import configuration
from helpers.resources import resource

from ionosenterprise.client import IonosEnterpriseService
from ionosenterprise.errors import ICNotFoundError


class TestImage(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        # Find an Ubuntu image for testing.
        for item in cls.client.list_images()['items']:
            if (configuration.IMAGE_NAME in item['properties']['name'] and
                    item['properties']['location'] == configuration.LOCATION):
                cls.image = item

    def test_list_images(self):
        images = self.client.list_images()

        assertRegex(self, images['items'][0]['id'], self.resource['uuid_match'])
        self.assertGreater(len(images), 0)
        self.assertEqual(images['items'][0]['type'], 'image')
        self.assertTrue(self, len(images['items']) > 0)

    def test_get_image(self):
        image = self.client.get_image(self.image['id'])

        self.assertEqual(image['type'], 'image')
        self.assertEqual(image['id'], self.image['id'])
        assertRegex(self, image['id'], self.resource['uuid_match'])
        self.assertGreater(len(image['properties']['name']), 0)
        self.assertIsNone(image['properties']['description'])
        self.assertGreater(image['properties']['size'], 0)
        self.assertNotEqual(image['properties']['name'], "")
        self.assertIn(image['properties']['location'], self.resource['locations'])
        self.assertIn(image['properties']['licenceType'], self.resource['licence_type'])
        self.assertIn(image['properties']['imageType'], ['HDD', 'CDROM'])
        self.assertIsInstance(image['properties']['imageAliases'], list)
        self.assertIsInstance(image['properties']['cpuHotPlug'], bool)
        self.assertIsInstance(image['properties']['cpuHotUnplug'], bool)
        self.assertIsInstance(image['properties']['ramHotPlug'], bool)
        self.assertIsInstance(image['properties']['ramHotUnplug'], bool)
        self.assertIsInstance(image['properties']['nicHotPlug'], bool)
        self.assertIsInstance(image['properties']['nicHotUnplug'], bool)
        self.assertIsInstance(image['properties']['discVirtioHotPlug'], bool)
        self.assertIsInstance(image['properties']['discVirtioHotUnplug'], bool)
        self.assertIsInstance(image['properties']['discScsiHotPlug'], bool)
        self.assertIsInstance(image['properties']['discScsiHotUnplug'], bool)
        self.assertIsInstance(image['properties']['public'], bool)

    def test_get_failure(self):
        try:
            self.client.get_image('00000000-0000-0000-0000-000000000000')
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])

    # A custom image would need to be uploaded and referenced to perform the
    # following tests. Skipping for now.

    # def test_delete_image(self):
    #     image = self.client.delete_image(image_id)
    #     self.assertTrue(image)

    # def test_update_image(self):
    #     image = self.client.update_image(
    #         image_id,
    #         name='New name')
    #    self.assertEqual(image['properties']['name'], 'New name')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python3

# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=pointless-string-statement,reimported,wrong-import-position

"""List LANs
"""
from ionosenterprise.client import IonosEnterpriseService

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'

client = IonosEnterpriseService(
    username='username', password='password')

lans = client.list_lans(datacenter_id=datacenter_id)

print(lans)

"""Create Complex LAN
"""
from ionosenterprise.client import IonosEnterpriseService, LAN  # noqa

lan_id = '4'
datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'

client = IonosEnterpriseService(
    username='username', password='password')

nics = ['<NIC-ID-1>', '<NIC-ID-2>']

i = LAN(
    name='public Lan 4',
    public=True,
    nics=nics)

response = client.create_lan(datacenter_id=datacenter_id, lan=i)

"""Create LAN
"""
from ionosenterprise.client import IonosEnterpriseService, LAN  # noqa

lan_id = '4'
datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'

client = IonosEnterpriseService(
    username='username', password='password')

i = LAN(
    name='public Lan 4',
    public=True)

response = client.create_lan(datacenter_id=datacenter_id, lan=i)

"""Get LAN Members
"""
from ionosenterprise.client import IonosEnterpriseService  # noqa

lan_id = '4'
datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'

client = IonosEnterpriseService(
    username='username', password='password')

members = client.get_lan_members(datacenter_id=datacenter_id,
                                 lan_id=lan_id)



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import warnings
import unittest
import time

from six import assertRegex

from helpers import configuration
from helpers.resources import resource, check_detached_cdrom_gone

from ionosenterprise.client import Datacenter, Server, Volume, \
    NIC, FirewallRule, IonosEnterpriseService
from ionosenterprise.errors import ICError, ICNotFoundError


class TestServer(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        # Create test datacenter.
        cls.datacenter = cls.client.create_datacenter(
            datacenter=Datacenter(**cls.resource['datacenter']))
        cls.client.wait_for_completion(cls.datacenter)

        # Create test volume1.
        cls.volume1 = cls.client.create_volume(
            datacenter_id=cls.datacenter['id'],
            volume=Volume(**cls.resource['volume']))
        cls.client.wait_for_completion(cls.volume1)

        # Create test volume2 (attach volume test).
        cls.volume2 = cls.client.create_volume(
            datacenter_id=cls.datacenter['id'],
            volume=Volume(**cls.resource['volume']))
        cls.client.wait_for_completion(cls.volume2)

        # Create test server.
        server = Server(**cls.resource['server'])
        server.attach_volumes = [cls.volume1['id']]
        cls.server = cls.client.create_server(
            datacenter_id=cls.datacenter['id'],
            server=server)
        cls.client.wait_for_completion(cls.server)

        # Create test NIC.
        cls.nic = cls.client.create_nic(
            datacenter_id=cls.datacenter['id'],
            server_id=cls.server['id'],
            nic=NIC(**cls.resource['nic']))
        cls.client.wait_for_completion(cls.nic)

        # Find an Ubuntu image for testing.
        for item in cls.client.list_images()['items']:
            if (configuration.IMAGE_NAME in item['properties']['name'] and
                    item['properties']['location'] == configuration.LOCATION):
                cls.image = item
        # Find a cdrom image
        images = cls.client.list_images(depth=5)
        usedIndex = 0
        for index, image in enumerate(images['items']):
            if (image['metadata']['state'] == "AVAILABLE"
                    and image['properties']['public'] is True
                    and image['properties']['imageType'] == "CDROM"
                    and image['properties']['location'] == configuration.LOCATION
                    and image['properties']['licenceType'] == "LINUX"):
                if usedIndex == 0:
                    cls.test_image1 = image
                    usedIndex = index
                else:
                    cls.test_image2 = image
                    break
        # Create test cdrom
        cls.cdrom = cls.client.attach_cdrom(
            datacenter_id=cls.datacenter['id'],
            server_id=cls.server['id'],
            cdrom_id=cls.test_image1['id'])
        cls.client.wait_for_completion(cls.cdrom)

    @classmethod
    def tearDownClass(cls):
        cls.client.delete_datacenter(datacenter_id=cls.datacenter['id'])

    def test_list_servers(self):
        servers = self.client.list_servers(datacenter_id=self.datacenter['id'])

        self.assertGreater(len(servers), 0)
        self.assertEqual(servers['items'][0]['type'], 'server')
        self.assertTrue(self, len(servers['items']) > 0)
        assertRegex(self, servers['items'][0]['id'], self.resource['uuid_match'])

    def test_get_server(self):
        server = self.client.get_server(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id']
        )

        self.assertEqual(server['type'], 'server')
        self.assertEqual(server['id'], self.server['id'])
        self.assertEqual(server['properties']['name'], self.resource['server']['name'])
        self.assertEqual(server['properties']['cores'], self.resource['server']['cores'])
        self.assertEqual(server['properties']['ram'], self.resource['server']['ram'])
        self.assertEqual(server['properties']['availabilityZone'],
                         self.resource['server']['availability_zone'])
        self.assertEqual(server['properties']['cpuFamily'], self.resource['server']['cpu_family'])
        # assertRegex(self, server['properties']['bootVolume']['id'], self.resource['uuid_match'])

    def test_get_failure(self):
        try:
            self.client.get_server(
                datacenter_id=self.datacenter['id'],
                server_id='00000000-0000-0000-0000-000000000000')
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])

    def test_delete_server(self):
        server = self.client.create_server(
            datacenter_id=self.datacenter['id'],
            server=Server(**self.resource['server'])
        )
        self.client.wait_for_completion(server)

        response = self.client.delete_server(
            datacenter_id=self.datacenter['id'],
            server_id=server['id']
        )

        self.assertTrue(response)
        assertRegex(self, response['requestId'], self.resource['uuid_match'])

    def test_update_server(self):
        server = self.client.update_server(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'],
            name=self.resource['server']['name'] + ' RENAME')
        self.client.wait_for_completion(server)
        server = self.client.get_server(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id']
        )

        self.assertEqual(server['id'], self.server['id'])
        self.assertEqual(server['properties']['name'], self.resource['server']['name'] + ' RENAME')
        self.assertEqual(server['properties']['cores'], self.resource['server']['cores'])
        self.assertEqual(server['properties']['ram'], self.resource['server']['ram'])

    def test_create_server(self):
        # Use server created during server test setup
        assertRegex(self, self.server['id'], self.resource['uuid_match'])
        self.assertEqual(self.server['type'], 'server')
        self.assertEqual(self.server['properties']['name'], self.resource['server']['name'])
        self.assertEqual(self.server['properties']['cores'], self.resource['server']['cores'])
        self.assertEqual(self.server['properties']['ram'], self.resource['server']['ram'])
        self.assertEqual(self.server['properties']['availabilityZone'],
                         self.resource['server']['availability_zone'])
        self.assertEqual(self.server['properties']['cpuFamily'],
                         self.resource['server']['cpu_family'])
        # assertRegex(self, server['properties']['bootVolume']['id'], self.resource['uuid_match'])
        # self.assertIsNone(self.server['properties']['availabilityZone'])
        self.assertIsNone(self.server['properties']['vmState'])

    def test_create_failure(self):
        try:
            server = Server(
                name=self.resource['server']['name'],
                ram=self.resource['server']['ram']
            )
            self.client.create_server(datacenter_id=self.datacenter['id'], server=server)
        except ICError as e:
            self.assertIn(self.resource['missing_attribute_error'] % 'cores',
                          e.content[0]['message'])

    def test_create_composite(self):
        fwrule = FirewallRule(**self.resource['fwrule'])
        nic = NIC(firewall_rules=[fwrule], **self.resource['nic'])
        volume = Volume(image=self.image['id'],
                        image_password='secretpassword123',
                        ssh_keys=['ssh-rsa AAAAB3NzaC1'],
                        **self.resource['volume'])
        volume.availability_zone = 'ZONE_3'

        server = Server(
            nics=[nic],
            create_volumes=[volume],
            **self.resource['server'])

        composite_server = self.client.create_server(
            datacenter_id=self.datacenter['id'],
            server=server)
        self.client.wait_for_completion(composite_server, timeout=600)

        composite_server = self.client.get_server(
            datacenter_id=self.datacenter['id'],
            server_id=composite_server['id'])

        assertRegex(self, composite_server['id'], self.resource['uuid_match'])
        self.assertEqual(composite_server['properties']['name'], self.resource['server']['name'])
        self.assertEqual(composite_server['properties']['cores'], self.resource['server']['cores'])
        self.assertEqual(composite_server['properties']['ram'], self.resource['server']['ram'])
        self.assertEqual(composite_server['properties']['availabilityZone'], 'ZONE_1')
        self.assertIn(composite_server['properties']['vmState'], self.resource['vm_states'])
        self.assertGreater(len(composite_server['entities']['volumes']['items']), 0)
        self.assertGreater(len(composite_server['entities']['nics']['items']), 0)

    def test_start_server(self):
        server = self.client.start_server(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'])

        self.assertTrue(server)

    def test_stop_server(self):
        server = self.client.stop_server(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'])

        self.assertTrue(server)

    def test_reboot_server(self):
        server = self.client.reboot_server(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'])

        self.assertTrue(server)

    def test_get_attached_volumes(self):
        volumes = self.client.get_attached_volumes(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'])

        self.assertGreater(len(volumes['items']), 0)
        self.assertEqual(volumes['items'][0]['type'], 'volume')

    def test_get_attached_volume(self):
        volume = self.client.get_attached_volume(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'],
            volume_id=self.volume1['id'])

        self.assertEqual(volume['properties']['name'], self.resource['volume']['name'])
        self.assertEqual(volume['properties']['size'], self.resource['volume']['size'])
        self.assertEqual(volume['properties']['bus'], self.resource['volume']['bus'])
        self.assertEqual(volume['properties']['type'], self.resource['volume']['disk_type'])
        self.assertEqual(volume['properties']['licenceType'],
                         self.resource['volume']['licence_type'])
        self.assertIsNone(volume['properties']['image'])
        self.assertIsNone(volume['properties']['imagePassword'])
        self.assertFalse(volume['properties']['cpuHotPlug'])
        self.assertFalse(volume['properties']['ramHotPlug'])
        self.assertFalse(volume['properties']['nicHotPlug'])
        self.assertFalse(volume['properties']['nicHotUnplug'])
        self.assertFalse(volume['properties']['discVirtioHotPlug'])
        self.assertFalse(volume['properties']['discVirtioHotUnplug'])

    def test_attach_volume(self):
        volume = self.client.attach_volume(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'],
            volume_id=self.volume2['id'])
        self.client.wait_for_completion(volume)

        self.assertEqual(volume['id'], self.volume2['id'])
        self.assertEqual(volume['properties']['name'], self.resource['volume']['name'])
        self.assertEqual(volume['properties']['size'], self.resource['volume']['size'])
        self.assertEqual(volume['properties']['type'], self.resource['volume']['disk_type'])
        self.assertIsNone(volume['properties']['bus'])
        self.assertIsNone(volume['properties']['image'])

        self.client.detach_volume(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'],
            volume_id=self.volume2['id'])

    def test_detach_volume(self):
        volume = self.client.detach_volume(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'],
            volume_id=self.volume1['id'])

        self.assertTrue(volume)

    def test_list_cdroms(self):
        cdroms = self.client.get_attached_cdroms(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'])

        self.assertGreater(len(cdroms['items']), 0)

    def test_attach_cdrom(self):
        attached_cdrom = self.client.attach_cdrom(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'],
            cdrom_id=self.test_image2['id'])

        self.client.wait_for_completion(attached_cdrom, timeout=600)
        self.assertEqual(attached_cdrom['id'], self.test_image2['id'])
        self.assertEqual(attached_cdrom['properties']['name'],
                         self.test_image2['properties']['name'])

    def test_get_cdrom(self):
        attached_cdrom = self.client.attach_cdrom(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'],
            cdrom_id=self.test_image1['id'])

        self.client.wait_for_completion(attached_cdrom, timeout=600)
        cdrom = self.client.get_attached_cdrom(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'],
            cdrom_id=attached_cdrom['id'])
        self.assertEqual(cdrom['id'], attached_cdrom['id'])
        self.assertEqual(cdrom['properties']['name'], attached_cdrom['properties']['name'])

    def test_detach_cdrom(self):
        detached_cd = self.client.detach_cdrom(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'],
            cdrom_id=self.cdrom['id'])
        time.sleep(15)

        self.assertTrue(detached_cd)

        try:
            check_detached_cdrom_gone(self)
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])


if __name__ == '__main__':
    unittest.main()



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class group:

    @IonosCoreProxy.process_response
    def list_groups(self, depth=1):
        """
        Retrieves a list of all groups.

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """
        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_groups_get_with_http_info(depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def get_group(self, group_id, depth=1):
        """
        Retrieves a single group by ID.

        :param      group_id: The unique ID of the group.
        :type       group_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """

        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_groups_find_by_id_with_http_info(group_id, depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def create_group(self, group):
        """
        Creates a new group and set group privileges.

        :param      group: The group object to be created.
        :type       group: ``dict``

        """

        group = ionoscloud.models.Group(
            properties=self._create_group_dict(group)['properties']
        )
        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_groups_post_with_http_info(group, response_type='object')

    @IonosCoreProxy.process_response
    def update_group(self, group_id, **kwargs):
        """
        Updates a group.

        :param      group_id: The unique ID of the group.
        :type       group_id: ``str``

        """
        properties = {}

        # make the key camel-case transformable
        if 'create_datacenter' in kwargs:
            kwargs['create_data_center'] = kwargs.pop('create_datacenter')

        for attr, value in kwargs.items():
            properties[self._underscore_to_camelcase(attr)] = value

        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_groups_put_with_http_info(
                group_id,
                ionoscloud.models.Group(
                    properties=properties
                ),
                response_type='object')

    @IonosCoreProxy.process_response
    def delete_group(self, group_id):
        """
        Removes a group.

        :param      group_id: The unique ID of the group.
        :type       group_id: ``str``

        """
        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_groups_delete_with_http_info(group_id)



class LAN(object):
    """
    This is the main class for managing LAN resources.
    """

    def __init__(self, name=None, public=None, nics=None, pcc_id=None):
        """
        LAN class initializer.

        :param      name: The name of the LAN.
        :type       name: ``str``

        :param      public: Indicates if the LAN is public.
        :type       public: ``bool``

        :param      nics: A list of NICs
        :type       nics: ``list``

        :param      pcc_id: Unique identifier of the private cross connect the given
                    LAN is connected to if any
        :type       pcc_id: ``str``

        """
        if nics is None:
            nics = []
        self.name = name
        self.public = public
        self.nics = nics
        self.pcc = pcc_id

    def __repr__(self):
        return (('<LAN: name=%s, public=%s, nics=%s, pcc_id=%s> ...>')
                % (self.name, str(self.public), str(self.nics), str(self.pcc)))



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class k8s:
    @IonosCoreProxy.process_response
    def get_k8s_cluster(self, k8s_cluster_id):
        """
        Retrieves a kubernetes cluster by its ID.

        :param      k8s_cluster_id: The unique ID of the cluster.
        :type       k8s_cluster_id: ``str``

        """

        return self.get_api_instance(ionoscloud.KubernetesApi)\
            .k8s_find_by_cluster_id_with_http_info(k8s_cluster_id, response_type='object')

    @IonosCoreProxy.process_response
    def list_k8s_clusters(self, depth=1):
        """
        Retrieves the list of kubernetes clusters.

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """
        return self.get_api_instance(ionoscloud.KubernetesApi)\
            .k8s_get_with_http_info(depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def create_k8s_cluster(self, name):
        """
        Creates a Kubernets cluster.

        :param      name: A Kubernetes Cluster Name. Valid Kubernetes
                          Cluster name must be 63 characters or less
                          and must be empty or begin and end with an
                          alphanumeric character ([a-z0-9A-Z]) with
                          dashes (-), underscores (_), dots (.), and
                          alphanumerics between.
        :type       name: ``str``

        """

        kubernetesCluster = ionoscloud.models.KubernetesCluster(
            properties={
                'name': name
            }
        )
        return self.get_api_instance(ionoscloud.KubernetesApi)\
            .k8s_post_with_http_info(kubernetesCluster, response_type='object')

    @IonosCoreProxy.process_response
    def delete_k8s_cluster(self, k8s_cluster_id):
        """
        Removes a kubernetes cluster.

        :param      k8s_cluster_id: The unique ID of the cluster.
        :type       k8s_cluster_id: ``str``

        """

        return self.get_api_instance(ionoscloud.KubernetesApi)\
            .k8s_delete_with_http_info(k8s_cluster_id)

    @IonosCoreProxy.process_response
    def update_k8s_cluster(self, k8s_cluster_id, **kwargs):
        """
        Replace all properties of a kubernetes cluster.

        :param      k8s_cluster_id: The unique ID of the cluster.
        :type       k8s_cluster_id: ``str``

        """

        data = {}

        for attr, value in kwargs.items():
            data[self._underscore_to_camelcase(attr)] = value

        kubernetesCluster = ionoscloud.models.KubernetesCluster(
            properties=data
        )
        return self.get_api_instance(ionoscloud.KubernetesApi)\
            .k8s_put_with_http_info(k8s_cluster_id, kubernetesCluster, response_type='object')



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
import warnings

from helpers import configuration
from helpers.resources import resource

from ionosenterprise.client import IonosEnterpriseService, Datacenter, Volume
from ionosenterprise.errors import ICError, ICNotAuthorizedError,\
    ICNotFoundError, ICValidationError


class TestErrors(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)
        cls.datacenter = cls.client.create_datacenter(
            datacenter=Datacenter(**cls.resource['datacenter']))

    @classmethod
    def tearDownClass(cls):
        cls.client.delete_datacenter(datacenter_id=cls.datacenter['id'])

    def test_ic_not_found(self):
        try:
            self.client.get_datacenter("fake_id")
        except ICError as err:
            self.assertTrue(isinstance(err, ICNotFoundError))

    def test_ic_unauthorized_error(self):
        try:
            self.client = IonosEnterpriseService(
                username=configuration.USERNAME + "1",
                password=configuration.PASSWORD,
                headers=configuration.HEADERS)
            self.client.list_datacenters()

        except ICError as err:
            self.assertTrue(isinstance(err, ICNotAuthorizedError))

    def test_ic_validation_error(self):
        try:
            i = Volume(
                name='Explicitly created volume',
                size=5,
                disk_type='HDD',
                image='fake_image_id',
                bus='VIRTIO')
            self.client.create_volume(datacenter_id=self.datacenter['id'], volume=i)
        except ICError as err:
            self.assertTrue(isinstance(err, ICValidationError))


if __name__ == '__main__':
    unittest.main()



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class cdrom:
    @IonosCoreProxy.process_response
    def get_attached_cdroms(self, datacenter_id, server_id, depth=1):
        """
        Retrieves a list of CDROMs attached to the server.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      server_id: The unique ID of the server.
        :type       server_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """

        return self.get_api_instance(ionoscloud.ServerApi)\
            .datacenters_servers_cdroms_get_with_http_info(
                datacenter_id, server_id, depth=depth, response_type='object'
            )

    @IonosCoreProxy.process_response
    def get_attached_cdrom(self, datacenter_id, server_id, cdrom_id):
        """
        Retrieves an attached CDROM.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      server_id: The unique ID of the server.
        :type       server_id: ``str``

        :param      cdrom_id: The unique ID of the CDROM.
        :type       cdrom_id: ``str``

        """
        return self.get_api_instance(ionoscloud.ServerApi)\
            .datacenters_servers_cdroms_find_by_id_with_http_info(datacenter_id, server_id,
                                                                  cdrom_id, response_type='object')

    @IonosCoreProxy.process_response
    def attach_cdrom(self, datacenter_id, server_id, cdrom_id):
        """
        Attaches a CDROM to a server.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      server_id: The unique ID of the server.
        :type       server_id: ``str``

        :param      cdrom_id: The unique ID of the CDROM.
        :type       cdrom_id: ``str``

        """
        image = ionoscloud.Image(id=cdrom_id)
        return self.get_api_instance(ionoscloud.ServerApi)\
            .datacenters_servers_cdroms_post_with_http_info(datacenter_id, server_id,
                                                            image, response_type='object')

    @IonosCoreProxy.process_response
    def detach_cdrom(self, datacenter_id, server_id, cdrom_id):
        """
        Detaches a volume from a server.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      server_id: The unique ID of the server.
        :type       server_id: ``str``

        :param      cdrom_id: The unique ID of the CDROM.
        :type       cdrom_id: ``str``

        """
        return self.get_api_instance(ionoscloud.ServerApi) \
            .datacenters_servers_cdroms_delete_with_http_info(datacenter_id, server_id,
                                                              cdrom_id,
                                                              response_type='object')



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
import uuid
import warnings

from helpers import configuration
from helpers.resources import resource

from ionosenterprise.client import IonosEnterpriseService, User


class TestS3key(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        user = User(
            firstname="TEST FIRST NAME",
            lastname="TEST FIRST NAME",
            email="%s@TEST-EMAIL.COM" % uuid.uuid1(),
            password="TEST PASSWORD"
        )
        # Create user
        cls.user = cls.client.create_user(user)
        cls.client.wait_for_completion(cls.user)
        # Create test s3keys.
        cls.s3key1 = cls.client.create_s3key(cls.user['id'])
        cls.s3key2 = cls.client.create_s3key(cls.user['id'])

    @classmethod
    def tearDownClass(cls):
        cls.client.delete_s3key(cls.user['id'], cls.s3key1['id'])
        cls.client.delete_user(cls.user['id'])

    def test_list_s3keys(self):
        s3keys = self.client.list_s3keys(self.user['id'])
        self.assertGreater(len(s3keys), 0)
        # one inactive key is created when user is created
        self.assertEqual(len(s3keys['items']), 2)

    def test_get_s3key(self):
        s3key = self.client.get_s3key(self.user['id'], self.s3key1['id'])
        self.assertTrue('id' in s3key)
        self.assertEqual(s3key['type'], 's3key')

    def test_delete_s3keys(self):
        response = self.client.delete_s3key(self.user['id'], self.s3key2['id'])
        self.assertTrue('requestId' in response)

    def test_update_s3key(self):
        s3key = self.client.update_s3key(
            self.user['id'], self.s3key1['id'],
            active=False
        )
        self.assertFalse(s3key['properties']['active'])

    def test_get_s3ssourl(self):
        ssoUrl = self.client.get_s3ssourl(self.user['id'])
        self.assertTrue('ssoUrl' in ssoUrl)



#!/usr/bin/python3

# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=pointless-string-statement,reimported,wrong-import-position

'''
This example will do the following:

- create a datacenter (name and location in variables datacenter_name and datacenter_location)
- wait for the datacenter to be active
- create a k8s cluster (cluster name in variable cluster_name)
- wait for the k8s cluster to be active
- retrieve the k8s cluster configuration
- create a k8s nodepool in the newly created k8s cluster
  (name and properties in variables pool_name and pool_properties)
- wait for the k8s nodepool to be active
- delete the k8s nodepool
- wait for the k8s nodepool to be deleted
- delete the k8s cluster
- wait for the k8s cluster to be deleted
- delete the datacenter
- wait for the datacenter to be deleted
'''

import os
import json
from ionosenterprise.client import IonosEnterpriseService, Datacenter


# ************* START CONFIG used for test **************
# ************* Update as needed           **************

datacenter_name = 'datacenter1'
datacenter_location = 'de/fra'
cluster_name = 'cluster1'
pool_name = 'pool1'
pool_properties = {
  'node_count': 4,
  'cpu_family': 'AMD_OPTERON',
  'cores_count': 2,
  'ram_size': 4096,
  'availability_zone': 'AUTO',
  'storage_type': 'SSD',
  'storage_size': 100
}

# ************* END CONFIG used for test   **************

username = os.getenv('IONOS_USERNAME', 'my_username')
password = os.getenv('IONOS_PASSWORD', 'my_password')

client = IonosEnterpriseService(
  username=username,
  password=password,
)

print('Creating datacenter named: %s in location: %s' % (datacenter_name, datacenter_location))
datacenter_props = {
  'name': datacenter_name,
  'location': datacenter_location
}
datacenter = client.create_datacenter(datacenter=Datacenter(**datacenter_props))
print('Request completed!')

print('Waiting for the datacenter to be active!')
datacenters = client.wait_for(
  fn_request=client.list_datacenters(),
  fn_check=lambda r: list(filter(
      lambda e: e['properties']['name'] == datacenter_name,
      r['items']
    ))[0]['metadata']['state'] == 'AVAILABLE',
  console_print='.',
  scaleup=10000
)
print('Datacenter active!')

datacenter_id = datacenter['id']

print('Using datacenter with ID: %s' % datacenter_id)

print('Creating K8S cluster named: %s' % cluster_name)
resp = client.create_k8s_cluster(cluster_name)
print('Request completed!')

print('Waiting for the cluster to be active!')
clusters = client.wait_for(
  fn_request=client.list_k8s_clusters(),
  fn_check=lambda r: list(filter(
      lambda e: e['properties']['name'] == cluster_name,
      r['items']
    ))[0]['metadata']['state'] == 'ACTIVE',
  console_print='.',
  scaleup=10000
)
print('Cluster active!')

my_cluster = list(filter(lambda e: e['properties']['name'] == cluster_name, clusters['items']))[0]
print(json.dumps(my_cluster, indent=4))

print('Getting cluster config!')
resp = client.get_k8s_config(my_cluster['id'])
print('Request completed!')

print(json.dumps(resp, indent=4))

print('Creating K8S nodepool named: %s' % pool_name)
resp = client.create_k8s_cluster_nodepool(
  my_cluster['id'],
  name=pool_name,
  datacenter_id=datacenter_id,
  **pool_properties
)
print('Request completed!')

print('Waiting for the nodepool to be active!')
pools = client.wait_for(
  fn_request=lambda: client.list_k8s_cluster_nodepools(my_cluster['id']),
  fn_check=lambda r: list(filter(
      lambda e: e['properties']['name'] == pool_name,
      r['items']
    ))[0]['metadata']['state'] == 'ACTIVE',
  console_print='.',
  scaleup=10000
)
print('Nodepool active!')

my_pool = list(filter(lambda e: e['properties']['name'] == pool_name, pools['items']))[0]
print(json.dumps(my_pool, indent=4))

print('Deleting K8S nodepool named: %s' % pool_name)
client.delete_k8s_cluster_nodepool(my_cluster['id'], my_pool['id'])
print('Request completed!')

print('Waiting for the nodepool to be deleted!')
clusters = client.wait_for(
  fn_request=lambda: client.list_k8s_cluster_nodepools(my_cluster['id']),
  fn_check=lambda r: len(list(filter(
      lambda e: e['properties']['name'] == pool_name,
      r['items']
    ))) == 0,
  console_print='.',
  scaleup=10000
)
print('Nodepool deleted!')

print('Deleting K8S cluster named: %s' % cluster_name)
client.delete_k8s_cluster(my_cluster['id'])
print('Request completed!')

print('Waiting for the cluster to be deleted!')
clusters = client.wait_for(
  fn_request=client.list_k8s_clusters(),
  fn_check=lambda r: len(list(filter(
      lambda e: e['properties']['name'] == cluster_name,
      r['items']
    ))) == 0,
  console_print='.',
  scaleup=10000
)
print('Cluster deleted!')

print('Deleting datacenter named: %s in location: %s' % (datacenter_name, datacenter_location))
client.delete_datacenter(datacenter_id)
print('Request completed!')

print('Waiting for the datacenter to be deleted!')
clusters = client.wait_for(
  fn_request=client.list_datacenters(),
  fn_check=lambda r: len(list(filter(
      lambda e: e['properties']['name'] == datacenter_name,
      r['items']
    ))) == 0,
  console_print='.',
  scaleup=10000
)
print('Datacenter deleted!')



import ionoscloud

from ionoscloud.models.backup_unit import BackupUnit
from ionoscloud.models.backup_unit_properties import BackupUnitProperties

from coreadaptor.IonosCoreProxy import IonosCoreProxy


class backupunit:

    @IonosCoreProxy.process_response
    def list_backupunits(self, depth=1):
        """
        You can retrieve a complete list of backup Units that you have access to.

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """
        return self.get_api_instance(ionoscloud.BackupUnitApi)\
            .backupunits_get_with_http_info(depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def create_backupunit(self, backupunit):
        """
        Create a Backup Unit. A Backup Unit is considered a resource like a virtual
        datacenter, IP Block, snapshot, etc.
        It shall be shareable via groups inside our User Management Feature

        :param      backupunit: The backupunit object.
        :type       backupunit: ``dict``

        """
        data = self._create_backupunit_dict(backupunit)

        return self.get_api_instance(ionoscloud.BackupUnitApi)\
            .backupunits_post_with_http_info(BackupUnit(**data), response_type='object')

    @IonosCoreProxy.process_response
    def get_backupunit(self, backupunit_id, depth=1):
        """
        Retrieve the details of an specific backup unit.

        :param      backupunit_id: Id of the backup unit.
        :type       backupunit_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """
        return self.get_api_instance(ionoscloud.BackupUnitApi)\
            .backupunits_find_by_id_with_http_info(backupunit_id,
                                                   depth=depth,
                                                   response_type='object')

    @IonosCoreProxy.process_response
    def update_backupunit(self, backupunit_id, **kwargs):
        """
        Partially modify a Backup Unit

        :param      backupunit_id: The unique ID of the backupunit.
        :type       backupunit_id: ``str``

        :param      kwargs: Fields to edit backupunit.
        :type       kwargs: ``dict``

        """
        data = {}

        for attr, value in kwargs.items():
            data[attr] = value

        backupUnitProperties = BackupUnitProperties(**data)

        return self.get_api_instance(ionoscloud.BackupUnitApi)\
            .backupunits_patch_with_http_info(
                backupunit_id, backupUnitProperties, response_type='object'
            )

    @IonosCoreProxy.process_response
    def update_backupunit_put(self, backupunit_id, **kwargs):
        """
        Modify a Backup Unit.

        :param      backupunit_id: The unique ID of the backupunit.
        :type       backupunit_id: ``str``

        :param      kwargs: Fields to edit backupunit.
        :type       kwargs: ``dict``

        """
        data = {}

        for attr, value in kwargs.items():
            data[attr] = value

        backupUnitProperties = BackupUnit(properties=BackupUnitProperties(**data))

        return self.get_api_instance(ionoscloud.BackupUnitApi)\
            .backupunits_put_with_http_info(backupunit_id,
                                            backupUnitProperties,
                                            response_type='object')

    @IonosCoreProxy.process_response
    def delete_backupunit(self, backupunit_id):
        """
        Delete a Backup Unit.

        :param      backupunit_id: The unique ID of the backupunit.
        :type       backupunit_id: ``str``

        """

        return self.get_api_instance(ionoscloud.BackupUnitApi)\
            .backupunits_delete_with_http_info(backupunit_id)

    @IonosCoreProxy.process_response
    def get_ssourl(self, backupunit_id):
        """
        Returns a single signon URL for the specified backup Unit.

        :param      backupunit_id: The unique ID of the backupunit.
        :type       backupunit_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """

        return self.get_api_instance(ionoscloud.BackupUnitApi)\
            .backupunits_ssourl_get_with_http_info(backupunit_id, response_type='object')



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class k8s_config:
    @IonosCoreProxy.process_response
    def get_k8s_config(self, k8s_cluster_id):
        """
        Retrieves a kubernetes cluster config by its ID.

        :param      k8s_cluster_id: The unique ID of the cluster.
        :type       k8s_cluster_id: ``str``

        """
        return self.get_api_instance(ionoscloud.KubernetesApi)\
            .k8s_kubeconfig_get_with_http_info(k8s_cluster_id, response_type='object')



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class location:

    @IonosCoreProxy.process_response
    def get_location(self, location_id, depth=0):
        """
        Retrieves a single location by ID.

        :param      location_id: The unique ID of the location.
        :type       location_id: ``str``

        """
        return self.get_api_instance(ionoscloud.LocationApi)\
            .locations_find_by_region_id_with_http_info(location_id, depth=depth,
                                                        response_type='object')

    @IonosCoreProxy.process_response
    def list_locations(self, depth=0):
        """
        Retrieves a list of locations available in the account.

        """
        return self.get_api_instance(ionoscloud.LocationApi)\
            .locations_get_with_http_info(depth=depth, response_type='object')



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class share:
    @IonosCoreProxy.process_response
    def list_shares(self, group_id, depth=1):
        """
        Retrieves a list of all shares though a group.

        :param      group_id: The unique ID of the group.
        :type       group_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """

        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_groups_shares_get_with_http_info(group_id, depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def get_share(self, group_id, resource_id, depth=1):
        """
        Retrieves a specific resource share available to a group.

        :param      group_id: The unique ID of the group.
        :type       group_id: ``str``

        :param      resource_id: The unique ID of the resource.
        :type       resource_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """

        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_groups_shares_find_by_resource_id_with_http_info(
            group_id, resource_id, depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def add_share(self, group_id, resource_id, **kwargs):
        """
        Shares a resource through a group.

        :param      group_id: The unique ID of the group.
        :type       group_id: ``str``

        :param      resource_id: The unique ID of the resource.
        :type       resource_id: ``str``

        """
        properties = {}

        for attr, value in kwargs.items():
            properties[self._underscore_to_camelcase(attr)] = value

        resource = ionoscloud.models.GroupShare(
            properties=properties
        )

        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_groups_shares_post_with_http_info(group_id, resource_id, resource,
                                                  response_type='object')

    @IonosCoreProxy.process_response
    def update_share(self, group_id, resource_id, **kwargs):
        """
        Updates the permissions of a group for a resource share.

        :param      group_id: The unique ID of the group.
        :type       group_id: ``str``

        :param      resource_id: The unique ID of the resource.
        :type       resource_id: ``str``

        """
        properties = {}

        for attr, value in kwargs.items():
            properties[self._underscore_to_camelcase(attr)] = value

        group_share = ionoscloud.models.GroupShare(
            properties=properties
        )
        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_groups_shares_put_with_http_info(
            group_id, resource_id, group_share, response_type='object')

    @IonosCoreProxy.process_response
    def delete_share(self, group_id, resource_id):
        """
        Removes a resource share from a group.

        :param      group_id: The unique ID of the group.
        :type       group_id: ``str``

        :param      resource_id: The unique ID of the resource.
        :type       resource_id: ``str``

        """

        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_groups_shares_delete_with_http_info(group_id, resource_id)



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import warnings
import unittest

from six import assertRegex

from helpers import configuration
from helpers.resources import resource

from ionosenterprise.client import Datacenter, Volume, Snapshot, IonosEnterpriseService
from ionosenterprise.errors import ICNotFoundError


class TestSnapshot(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        # Create test datacenter.
        cls.datacenter = cls.client.create_datacenter(
            datacenter=Datacenter(**cls.resource['datacenter']))
        cls.client.wait_for_completion(cls.datacenter)

        # Create test volume
        volume = Volume(**cls.resource['volume'])
        cls.volume = cls.client.create_volume(
            datacenter_id=cls.datacenter['id'],
            volume=volume
        )

        cls.client.wait_for_completion(cls.volume)

        # Create test volume1
        volume1 = Volume(**cls.resource['volume'])
        cls.volume1 = cls.client.create_volume(
            datacenter_id=cls.datacenter['id'],
            volume=volume1
        )

        cls.client.wait_for_completion(cls.volume1)

        # Create test snapshot
        snapshot = Snapshot(**cls.resource['snapshot'])
        cls.snapshot1 = cls.client.create_snapshot(
            datacenter_id=cls.datacenter['id'],
            volume_id=cls.volume['id'],
            name=snapshot.name,
            description=snapshot.description)

        cls.client.wait_for_completion(cls.snapshot1)

        # Create test snapshot2
        cls.snapshot2 = cls.client.create_snapshot(
            datacenter_id=cls.datacenter['id'],
            volume_id=cls.volume['id'],
            name="python sdk test snapshot",
            description="snapshot test description")

        cls.client.wait_for_completion(cls.snapshot2)

    @classmethod
    def tearDownClass(cls):
        cls.client.delete_snapshot(snapshot_id=cls.snapshot1['id'])
        cls.client.delete_datacenter(datacenter_id=cls.datacenter['id'])

    def test_list_snapshots(self):
        snapshots = self.client.list_snapshots()

        self.assertGreater(len(snapshots['items']), 0)
        self.assertEqual(snapshots['items'][0]['type'], 'snapshot')

    def test_get_snapshot(self):
        snapshot = self.client.get_snapshot(snapshot_id=self.snapshot1['id'])

        self.assertEqual(snapshot['type'], 'snapshot')
        self.assertEqual(snapshot['id'], self.snapshot1['id'])
        self.assertEqual(snapshot['properties']['name'], self.resource['snapshot']['name'])
        self.assertTrue(snapshot['properties']['description'],
                        self.resource['snapshot']['description'])
        self.assertEqual(snapshot['properties']['location'], configuration.LOCATION)
        self.assertEqual(snapshot['properties']['size'], self.volume['properties']['size'])

    def test_delete_snapshot(self):
        snapshot = self.client.delete_snapshot(snapshot_id=self.snapshot2['id'])

        self.assertTrue(snapshot)
        assertRegex(self, snapshot['requestId'], self.resource['uuid_match'])

    def test_update_snapshot(self):
        snapshot = self.client.update_snapshot(
            snapshot_id=self.snapshot1['id'],
            name=self.resource['snapshot']['name'] + ' - RENAME',
            description=self.resource['snapshot']['description'] + ' - RENAME')

        self.client.wait_for_completion(snapshot)

        self.assertEqual(snapshot['type'], 'snapshot')
        self.assertEqual(snapshot['properties']['name'],
                         self.resource['snapshot']['name'] + ' - RENAME')
        self.assertEqual(snapshot['properties']['description'],
                         self.resource['snapshot']['description'] + ' - RENAME')

    def test_create_snapshot(self):
        self.assertEqual(self.snapshot1['type'], 'snapshot')
        self.assertEqual(self.snapshot1['properties']['name'],
                         self.resource['snapshot']['name'])
        self.assertEqual(self.snapshot1['properties']['description'],
                         self.resource['snapshot']['description'])

    def test_get_failure(self):
        try:
            self.client.get_snapshot('00000000-0000-0000-0000-000000000000')
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])

    def test_create_failure(self):
        try:
            self.client.create_snapshot(
                datacenter_id='00000000-0000-0000-0000-000000000000',
                volume_id=self.volume['id'],
                name=self.resource['snapshot']['name'])
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# encoding: utf-8

# Copyright 2016-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=broad-except,global-statement

'''
pb_datacenter_inventory -- dump inventory of your Ionos Enterprise data centers to CSV files

pb_datacenter_inventory is a sample script to get an inventory overview
of your data centers via Ionos Enterprise REST-API.

It collects some basic information on
- reserved ip blocks:
    Location,RscType,RscID,State,Size,IP addresses
- images and snapshots
    Visibility(public|private),Location,RscType,SubType(CDROM|HDD),RscID,
    RscName,State,LicType,Size,Created,Modified
- servers
    DCID,DCName,Location,RscType,RscID,RscName,State,LicType,Cores,RAM,
    # NICs,# Volumes,(Total) Storage,---,Created,Modified
  and storage
    DCID,DCName,Location,RscType,RscID,RscName,State,LicType,---,---,---,---,
    (Total) Storage,Connected to,Created,Modified
- networking infrastructure (LANs/NICs)
    DCID,DCName,Location,LAN ID,LAN name,public?,State,# NICs,NIC ID,
    MAC address,DHCP?,IP(s),NIC name,Firewall?,Connected to,ID,Name
from all your datacenters.

This data is written to a resource specific CSV file which can be used
for reports or automatic post processing.


@author:     Jürgen Buchhammer

@copyright:  2016 IONOS. All rights reserved.

@license:    license

@deffield    updated: Updated
'''

import sys
import os
import traceback
import re
import pprint

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
from getpass import getpass

import csv

from ionosenterprise.client import IonosEnterpriseService

__all__ = []
__version__ = 0.2
__date__ = '2016-01-15'
__updated__ = '2016-01-15'

verbose = 0
DEBUG = 1


def pp(value):
    """
    Returns a pretty print string of the given value.

    @return: pretty print string
    @rtype: str
    """

    pretty_printer = pprint.PrettyPrinter(indent=4)
    return pretty_printer.pformat(value)


class CLIError(Exception):
    '''Generic exception to raise and log different fatal errors.'''

    def __init__(self, msg):
        super(CLIError).__init__(type(self))
        self.msg = "E: %s" % msg

    def __str__(self):
        return self.msg

    def __unicode__(self):
        return self.msg


def get_dc_inventory(pbclient, dc=None):
    ''' gets inventory of one data center'''
    if pbclient is None:
        raise ValueError("argument 'pbclient' must not be None")
    if dc is None:
        raise ValueError("argument 'dc' must not be None")
    dc_inv = []   # inventory list to return
    dcid = dc['id']
    # dc_data contains dc specific columns
    dc_data = [dcid, dc['properties']['name'], dc['properties']['location']]
    # first get the servers
    # this will build a hash to relate volumes to servers later
    # depth 3 is enough to get into volume/nic level plus details
    servers = pbclient.list_servers(dcid, 3)
    print("found %i servers in data center %s" % (len(servers['items']), dc['properties']['name']))
    if verbose > 2:
        print(str(servers))
    # this will build a hash to relate volumes to servers later
    bound_vols = dict()   # hash volume-to-server relations
    for server in servers['items']:
        if verbose > 2:
            print("SERVER: %s" % str(server))
        serverid = server['id']
        # server_data contains server specific columns for later output
        server_data = [
            server['type'], serverid, server['properties']['name'],
            server['metadata']['state']
        ]
        # OS is determined by boot device (volume||cdrom), not a server property.
        # Might even be unspecified
        bootOS = "NONE"
        bootdev = server['properties']['bootVolume']
        if bootdev is None:
            bootdev = server['properties']['bootCdrom']
            print("server %s has boot device %s" % (serverid, "CDROM"))
        if bootdev is None:
            print("server %s has NO boot device" % (serverid))
        else:
            bootOS = bootdev['properties']['licenceType']
        server_data += [bootOS, server['properties']['cores'], server['properties']['ram']]
        server_vols = server['entities']['volumes']['items']
        n_volumes = len(server_vols)
        total_disk = 0
        licence_type = ""
        for vol in server_vols:
            total_disk += vol['properties']['size']
            licence_type = str(vol['properties']['licenceType'])
            bound_vols[vol['id']] = serverid
            if verbose:
                print("volume %s is connected to %s w/ OS %s" % (
                    vol['id'], bound_vols[vol['id']], licence_type))
        server_nics = server['entities']['nics']['items']
        n_nics = len(server_nics)
        server_data += [
            n_nics, n_volumes, total_disk, "",
            server['metadata']['createdDate'], server['metadata']['lastModifiedDate']
        ]
        dc_inv.append(dc_data + server_data)
    # end for(servers)

    # and now the volumes...
    volumes = pbclient.list_volumes(dcid, 2)   # depth 2 gives max. details
    for volume in volumes['items']:
        if verbose > 2:
            print("VOLUME: %s" % str(volume))
        volid = volume['id']
        vol_data = [
            volume['type'], volid, volume['properties']['name'], volume['metadata']['state'],
            volume['properties']['licenceType'], "", "", "", "", volume['properties']['size']
        ]
        connect = 'NONE'
        if volid in bound_vols:
            connect = bound_vols[volid]
        vol_data += [
            connect, volume['metadata']['createdDate'], volume['metadata']['lastModifiedDate']
        ]
        dc_inv.append(dc_data + vol_data)
    # end for(volumes)
    return dc_inv
# end get_dc_inventory()


def get_images(pbclient):
    if pbclient is None:
        raise ValueError("argument 'pbclient' must not be None")
    print("getting images..")
    images = pbclient.list_images()
    print("found %i images" % len(images['items']))
    img_inv = []
    for image in images['items']:
        if verbose > 2:
            print("IMAGE: %s" % str(image))
        img_data = [
            ('public' if image['properties']['public'] else 'private'),
            image['properties']['location'], image['type'],
            image['properties']['imageType'], image['id'], image['properties']['name'],
            image['metadata']['state'], image['properties']['licenceType'],
            image['properties']['size'],
            image['metadata']['createdDate'], image['metadata']['lastModifiedDate']
        ]
        img_inv.append(img_data)
    return img_inv
# end get_images()


def get_snapshots(pbclient):
    if pbclient is None:
        raise ValueError("argument 'pbclient' must not be None")
    print("getting snapshots..")
    snapshots = pbclient.list_snapshots()
    print("found %i snapshots" % len(snapshots['items']))
    snap_inv = []
    for snap in snapshots['items']:
        if verbose > 2:
            print("SNAPSHOT: %s" % str(snap))
        snap_data = [
            "private", snap['properties']['location'],
            snap['type'], "HDD", snap['id'], snap['properties']['name'],
            snap['metadata']['state'], snap['properties']['licenceType'],
            snap['properties']['size'],
            snap['metadata']['createdDate'], snap['metadata']['lastModifiedDate']
        ]
        # print("SNAPSHOT: %s" % str(snap_data))
        snap_inv.append(snap_data)
    # end for(snapshots)
    return snap_inv
# end get_snapshots()


def get_ipblocks(pbclient):
    if pbclient is None:
        raise ValueError("argument 'pbclient' must not be None")
    print("getting IP blocks..")
    ipblocks = pbclient.list_ipblocks()
    print("found %i IP blocks" % len(ipblocks['items']))
    ip_inv = []
    if verbose > 1:
        print(str(ipblocks))
    for block in ipblocks['items']:
        ip_data = [
            block['properties']['location'], block['type'], block['id'],
            block['metadata']['state'], block['properties']['size']
        ]
        ip_data.extend(block['properties']['ips'])
        # print str(ip_data)
        ip_inv.append(ip_data)
    # end for(ipblocks)
    return ip_inv
# end get_ipblocks()


def get_dc_network(pbclient, dc=None):
    ''' gets inventory of one data center'''
    if pbclient is None:
        raise ValueError("argument 'pbclient' must not be None")
    if dc is None:
        raise ValueError("argument 'dc' must not be None")
    print("getting networks..")
    dcid = dc['id']
    # dc_data contains dc specific columns
    dc_data = [dcid, dc['properties']['name'], dc['properties']['location']]
    lbs = pbclient.list_loadbalancers(dcid, 2)
    # build lookup hash for loadbalancer's ID->name
    lbnames = dict([(lb['id'], lb['properties']['name']) for lb in lbs['items']])
    if verbose > 2:
        print("LBs: %s" % (str(lbs)))
    lans = pbclient.list_lans(dcid, 3)
    lan_inv = []
    # lookup hash for server's ID->name
    servernames = dict()
    for lan in lans['items']:
        if verbose > 1:
            print("LAN: %s" % str(lan))
        lan_data = dc_data + [
            "LAN "+lan['id'], lan['properties']['name'], lan['properties']['public'],
            lan['metadata']['state']
        ]
        nics = lan['entities']['nics']['items']
        lan_data.append(len(nics))
        if nics:
            for nic in nics:
                nic_props = nic['properties']
                # get the serverid of this nic by href
                # !!! HUUUUH this might also be a loadbalancer ID,
                # although it's '/servers/<id>/...' !!!
                serverid = re.sub(r'^.*servers/([^/]+)/nics.*', r'\1', nic['href'])
                if serverid in lbnames:
                    servertype = "LB"
                    servername = lbnames[serverid]
                    print("server entry for %s is LOADBALANCER %s" % (serverid, servername))
                else:
                    servertype = "Server"
                    if serverid not in servernames:
                        if verbose:
                            print("add server entry for %s" % serverid)
                        server = pbclient.get_server(dcid, serverid, 0)
                        servernames[serverid] = server['properties']['name']
                    servername = servernames[serverid]
                # end if/else(serverid)
                ips = [str(ip) for ip in nic_props['ips']]
                nic_data = [
                    nic['id'], nic_props['mac'], nic_props['dhcp'], ips, nic_props['name'],
                    nic_props['firewallActive'], servertype, serverid, servername
                ]
                lan_inv.append(lan_data+nic_data)
            # end for(nics)
        else:
            lan_inv.append(lan_data)
    # end for(lans)
    return lan_inv
# end get_networks()


def get_requests(pbclient):
    if pbclient is None:
        raise ValueError("argument 'pbclient' must not be None")
    print("getting requests..")
    requests = pbclient.list_requests()   # higher depth gives no more details (API-doc BUG)
    print("found %i requests" % len(requests['items']))
    if verbose:
        print("global: %s" % str(requests))
    for req in requests['items']:
        reqid = req['id']
        # False: get request (w/ body), True: get request status (DONE,..)
        req_data = pbclient.get_request(reqid)
        # False: get request (w/ body), True: get request status (DONE,..)
        req_stat = pbclient.get_request(reqid, True)
        if verbose > 2:
            print("REQ: %s\n     %s" % (str(req_data), str(req_stat)))
        if verbose > 1:
            print(
                "request %s|%s|%s" % (
                    str(reqid), str(req_data['metadata']), str(req_stat['metadata'])
                )
            )
    # end for(requests)
# end get_requests()


def main(argv=None):                # IGNORE:C0111
    '''Command line options.'''

    if argv is None:
        argv = sys.argv
    else:
        sys.argv.extend(argv)

    program_name = os.path.basename(sys.argv[0])
    program_version = "v%s" % __version__
    program_build_date = str(__updated__)
    program_version_message = '%%(prog)s %s (%s)' % (program_version, program_build_date)
    program_shortdesc = __import__('__main__').__doc__.split("\n")[1]
    program_license = '''%s

  Created by J.Buchhammer on %s.
  Copyright 2016 IONOS. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc, str(__date__))

    try:
        # Setup argument parser
        parser = ArgumentParser(
            description=program_license,
            formatter_class=RawDescriptionHelpFormatter)
        parser.add_argument(
            '-u', '--user', dest='user', required=True, help='the login name')
        parser.add_argument(
            '-p', '--password', dest='password', help='the login password')
        parser.add_argument(
            '-d', '--datacenter', '--datacenterid', dest='datacenterid', nargs='?', const='*',
            help='show server/storage of datacenter(s)')
        parser.add_argument(
            '-i', '--image', dest='show_images', action="store_true",
            help='show images and snapshots')
        parser.add_argument(
            '-b', '--ipblock', dest='show_ipblocks', action="store_true",
            help='show reserved IP blocks')
        parser.add_argument(
            '-n', '--network', dest='show_networks', action="store_true",
            help='show network assignments')
#        parser.add_argument(
#            '-r', '--request', dest='show_requests', action="store_true",
#            help='show requests')
        parser.add_argument(
            "-v", "--verbose", dest="verbose", action="count", default=0,
            help="set verbosity level [default: %(default)s]")
        parser.add_argument(
            '-V', '--version', action='version', version=program_version_message)

        # Process arguments
        args = parser.parse_args()
        global verbose
        verbose = args.verbose   # this is a global to be used in methods
        user = args.user
        password = args.password
        datacenterid = args.datacenterid

        print("Welcome to PB-API %s\n" % user)
        if password is None:
            password = getpass()
        if verbose > 0:
            print("Verbose mode on")
            print("using python ", sys.version_info)

        pbclient = IonosEnterpriseService(user, password)

        if datacenterid is not None:
            datacenters = {}
            if datacenterid == '*':
                # the default depth=1 is sufficient, higher values don't provide more details
                datacenters = pbclient.list_datacenters()
            else:
                datacenters['items'] = []
                datacenters['items'] = [pbclient.get_datacenter(datacenterid, 1)]
            if verbose > 1:
                print(pp(datacenters))
            print("retrieved %i datacenters " % len(datacenters['items']))

            # dump inventory to file
            with open("pb_datacenter_inventory.csv", 'w') as csvfile:
                csvwriter = csv.writer(csvfile, delimiter=';', lineterminator='\n')
                csvwriter.writerow([
                    'DCID', 'DCName', 'Loc', 'RscType', 'RscID', 'RscName', 'State', 'LicType',
                    'Cores', 'RAM', '# NICs', '# Volumes', '(Total) Storage', 'Connected to',
                    'Created', 'Modified'
                ])
                for dc in datacenters['items']:
                    try:
                        dc_inv = get_dc_inventory(pbclient, dc)
                        if verbose:
                            print("DC %s has %i inventory entries" % (dc['id'], len(dc_inv)))
                        for row in dc_inv:
                            csvwriter.writerow(row)
                    except Exception:
                        traceback.print_exc()
                        exit(2)
                # end for(datacenters)

        if args.show_images:
            with open("pb_datacenter_images.csv", 'w') as csvfile:
                csvwriter = csv.writer(csvfile, delimiter=';', lineterminator='\n')
                csvwriter.writerow([
                    'Visibility', 'Loc', 'RscType', 'SubType', 'RscID', 'RscName',
                    'State', 'LicType', 'Size', 'Created', 'Modified'
                ])
                img_inv = get_images(pbclient)
                for row in img_inv:
                    csvwriter.writerow(row)
                snap_inv = get_snapshots(pbclient)
                for row in snap_inv:
                    csvwriter.writerow(row)

        if args.show_ipblocks:
            with open("pb_datacenter_ipblocks.csv", 'w') as csvfile:
                csvwriter = csv.writer(csvfile, delimiter=';', lineterminator='\n')
                csvwriter.writerow([
                    'Loc', 'RscType', 'RscID', 'State', 'Size', 'IP addresses'])
                ipblocks = get_ipblocks(pbclient)
                for row in ipblocks:
                    csvwriter.writerow(row)

        # file is automatically closed after with block
        if args.show_networks:
            # the default depth=1 is sufficient, higher values don't provide more details
            datacenters = pbclient.list_datacenters()
            print("retrieved %i datacenters " % len(datacenters['items']))
            with open("pb_datacenter_networks.csv", 'w') as csvfile:
                csvwriter = csv.writer(csvfile, delimiter=';', lineterminator='\n')
                csvwriter.writerow([
                    'DCID', 'DCName', 'Loc',
                    'LAN ID', 'LAN name', 'public', 'State', '# NICs',
                    'NIC ID', 'MAC address', 'DHCP', 'IP(s)', 'NIC name', 'Firewall',
                    'Connected to', 'ID', 'Name'])

                for dc in datacenters['items']:
                    try:
                        dc_net = get_dc_network(pbclient, dc)
                        if verbose:
                            print("DC %s has %i network entries" % (dc['id'], len(dc_net)))
                        for row in dc_net:
                            csvwriter.writerow(row)
                    except Exception:
                        traceback.print_exc()
                        exit(2)
                # end for(datacenters)

        # just for fun:
#         if args.show_requests:
#             get_requests(pbclient)
        print("%s finished w/o errors" % program_name)
        return 0
    except KeyboardInterrupt:
        # handle keyboard interrupt
        return 0
    except Exception:
        traceback.print_exc()
        sys.stderr.write("\n" + program_name + ":  for help use --help\n")
        return 2


if __name__ == "__main__":
    sys.exit(main())



from .datacenter import Datacenter
from .server import Server
from .firewall import FirewallRule
from .group import Group
from .ipblock import IPBlock
from .lan import LAN
from .loadbalancer import LoadBalancer
from .nic import NIC
from .snapshot import Snapshot
from .user import User
from .volume import Volume
from .privatecrossconnect import PrivateCrossConnect
from .backupunit import BackupUnit

__all__ = [
    'Datacenter',
    'Server',
    'FirewallRule',
    'Group',
    'IPBlock',
    'LAN',
    'LoadBalancer',
    'NIC',
    'Snapshot',
    'User',
    'Volume',
    'PrivateCrossConnect',
    'BackupUnit'
]



class Server(object):
    """
    This is the main class for managing server resources.
    """

    def __init__(self, name=None, cores=None, ram=None, availability_zone=None,
                 boot_volume_id=None, boot_cdrom=None, cpu_family=None,
                 create_volumes=None, attach_volumes=None, nics=None):
        """
        Server class initializer.

        :param      name: The name of your server..
        :type       name: ``str``

        :param      cores: The number of cores for the server.
        :type       cores: ``str``

        :param      ram: The amount of memory for the server.
        :type       ram: ``str``

        :param      availability_zone: The availability zone for the server.
        :type       availability_zone: ``str``

        :param      boot_volume_id: The ID of the boot volume.
        :type       boot_volume_id: ``str``

        :param      boot_cdrom: Attach a CDROM.
        :type       boot_cdrom: ``str``

        :param      cpu_family: Set the desired CPU type.
        :type       cpu_family: ``str``

        :param      create_volumes: List of volume dicts to create.
        :type       create_volumes: ``list``

        :param      attach_volumes: List of volume IDs to attach.
        :type       attach_volumes: ``list``

        :param      nics: List of NIC dicts to create.
        :type       nics: ``list``

        """
        if create_volumes is None:
            create_volumes = []
        if attach_volumes is None:
            attach_volumes = []
        if nics is None:
            nics = []
        self.name = name
        self.cores = cores
        self.ram = ram
        self.availability_zone = availability_zone
        self.boot_volume_id = boot_volume_id
        self.boot_cdrom = boot_cdrom
        self.cpu_family = cpu_family
        self.create_volumes = create_volumes
        self.attach_volumes = attach_volumes
        self.nics = nics

    def __repr__(self):
        return (('<Server: name=%s, cores=%s, ram=%s, '
                 'availability_zone=%s, boot_volume_id=%s, '
                 'boot_cdrom=%s, ...>')
                % (self.name, self.cores, self.ram,
                   self.availability_zone, self.boot_volume_id, self.boot_cdrom))



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import warnings
import unittest

from helpers import configuration
from helpers.resources import resource

from ionosenterprise.client import IonosEnterpriseService, PrivateCrossConnect


class TestPcc(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        pccModel1 = PrivateCrossConnect(name="TEST NAME 1", description="TEST DESCRIPTION 1")
        cls.pcc1 = cls.client.create_pcc(pccModel1)
        pccModel2 = PrivateCrossConnect(name="TEST NAME 2", description="TEST DESCRIPTION 2")
        cls.pcc2 = cls.client.create_pcc(pccModel2)

    @classmethod
    def tearDownClass(cls):
        cls.client.delete_pcc(cls.pcc1['id'])

    def test_list_pccs(self):
        pccs = self.client.list_pccs()
        self.assertGreater(len(pccs), 0)
        self.assertGreater(len(pccs['items']), 1)

    def test_get_pcc(self):
        pcc = self.client.get_pcc(self.pcc1['id'])
        self.assertTrue('id' in pcc)
        self.assertEqual(pcc['type'], 'pcc')

    def test_delete_pccs(self):
        response = self.client.delete_pcc(self.pcc2['id'])
        self.assertTrue('requestId' in response)

    def test_update_pcc(self):
        pcc = self.client.update_pcc(self.pcc1['id'], name="TEST NAME 1 - UPDATED",
                                     description="TEST DESCRIPTION 1 - UPDATED")
        self.assertEqual(pcc['properties']['name'], "TEST NAME 1 - UPDATED")
        self.assertEqual(pcc['properties']['description'], "TEST DESCRIPTION 1 - UPDATED")



class IPBlock(object):
    def __init__(self, name=None, location=None, size=None):
        """
        IPBlock class initializer.

        :param      name: The name of the IP block.
        :type       name: ``str``

        :param      location: The location for the IP block.
        :type       location: ``str``

        :param      size: The number of IPs in the block.
        :type       size: ``str``

        """
        self.name = name
        self.location = location
        self.size = size

    def __repr__(self):
        return (('<IPBlock: location=%s, size=%s>')
                % (self.location, self.size))



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class firewall:
    @IonosCoreProxy.process_response
    def get_firewall_rule(self, datacenter_id, server_id, nic_id, firewall_rule_id):

        """
        Retrieves a single firewall rule by ID.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      server_id: The unique ID of the server.
        :type       server_id: ``str``

        :param      nic_id: The unique ID of the NIC.
        :type       nic_id: ``str``

        :param      firewall_rule_id: The unique ID of the firewall rule.
        :type       firewall_rule_id: ``str``

        """

        return self.get_api_instance(ionoscloud.NicApi) \
            .datacenters_servers_nics_firewallrules_find_by_id_with_http_info(
                datacenter_id,
                server_id,
                nic_id,
                firewall_rule_id,
                response_type='object')

    @IonosCoreProxy.process_response
    def get_firewall_rules(self, datacenter_id, server_id, nic_id, depth=1):
        """
        Retrieves a list of firewall rules available in the account.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      server_id: The unique ID of the server.
        :type       server_id: ``str``

        :param      nic_id: The unique ID of the NIC.
        :type       nic_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """

        return self.get_api_instance(ionoscloud.NicApi) \
            .datacenters_servers_nics_firewallrules_get_with_http_info(
            datacenter_id, server_id, nic_id, depth=depth, response_type='object'
        )

    @IonosCoreProxy.process_response
    def delete_firewall_rule(self, datacenter_id, server_id,
                             nic_id, firewall_rule_id):
        """
        Removes a firewall rule from the NIC.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      server_id: The unique ID of the server.
        :type       server_id: ``str``

        :param      nic_id: The unique ID of the NIC.
        :type       nic_id: ``str``

        :param      firewall_rule_id: The unique ID of the firewall rule.
        :type       firewall_rule_id: ``str``

        """

        return self.get_api_instance(ionoscloud.NicApi) \
            .datacenters_servers_nics_firewallrules_delete_with_http_info(
            datacenter_id, server_id, nic_id, firewall_rule_id)

    @IonosCoreProxy.process_response
    def create_firewall_rule(self, datacenter_id, server_id, nic_id, firewall_rule):
        """
        Creates a firewall rule on the specified NIC and server.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      server_id: The unique ID of the server.
        :type       server_id: ``str``

        :param      nic_id: The unique ID of the NIC.
        :type       nic_id: ``str``

        :param      firewall_rule: A firewall rule dict.
        :type       firewall_rule: ``dict``

        """
        properties = {
            "name": firewall_rule.name
        }

        if firewall_rule.protocol:
            properties['protocol'] = firewall_rule.protocol

        # Optional Properties
        if firewall_rule.source_mac:
            properties['sourceMac'] = firewall_rule.source_mac

        if firewall_rule.source_ip:
            properties['sourceIp'] = firewall_rule.source_ip

        if firewall_rule.target_ip:
            properties['targetIp'] = firewall_rule.target_ip

        if firewall_rule.port_range_start:
            properties['portRangeStart'] = firewall_rule.port_range_start

        if firewall_rule.port_range_end:
            properties['portRangeEnd'] = firewall_rule.port_range_end

        if firewall_rule.icmp_type:
            properties['icmpType'] = firewall_rule.icmp_type

        if firewall_rule.icmp_code:
            properties['icmpCode'] = firewall_rule.icmp_code

        firewallRule = ionoscloud.models.FirewallRule(
            properties=properties
        )
        return self.get_api_instance(ionoscloud.NicApi) \
            .datacenters_servers_nics_firewallrules_post_with_http_info(
            datacenter_id, server_id, nic_id, firewallRule, response_type='object')

    @IonosCoreProxy.process_response
    def update_firewall_rule(self, datacenter_id, server_id,
                             nic_id, firewall_rule_id, **kwargs):
        """
        Updates a firewall rule.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      server_id: The unique ID of the server.
        :type       server_id: ``str``

        :param      nic_id: The unique ID of the NIC.
        :type       nic_id: ``str``

        :param      firewall_rule_id: The unique ID of the firewall rule.
        :type       firewall_rule_id: ``str``

        """
        data = {}

        for attr, value in kwargs.items():
            data[self._underscore_to_camelcase(attr)] = value

            if attr == 'source_mac':
                data['sourceMac'] = value
            elif attr == 'source_ip':
                data['sourceIp'] = value
            elif attr == 'target_ip':
                data['targetIp'] = value
            elif attr == 'port_range_start':
                data['portRangeStart'] = value
            elif attr == 'port_range_end':
                data['portRangeEnd'] = value
            elif attr == 'icmp_type':
                data['icmpType'] = value
            elif attr == 'icmp_code':
                data['icmpCode'] = value
            else:
                data[self._underscore_to_camelcase(attr)] = value

        return self.get_api_instance(ionoscloud.NicApi) \
            .datacenters_servers_nics_firewallrules_patch_with_http_info(
            datacenter_id, server_id, nic_id, firewall_rule_id, data, response_type='object')



class dicts:
    @staticmethod
    def _create_lan_dict(lan):
        items = []
        entities = dict()

        properties = {
            "name": lan.name
        }

        # Optional Properties
        if lan.public is not None:
            properties['public'] = str(lan.public).lower()

        if lan.pcc is not None:
            properties['pcc'] = lan.pcc

        if lan.nics:
            for nic in lan.nics:
                nics_properties = {
                    "id": nic
                }
                items.append(nics_properties)

            item_entities = {
                "items": items
            }

            nics_entities = {
                "nics": item_entities
            }

            entities.update(nics_entities)

        if not entities:
            raw = {
                "properties": properties,
            }
        else:
            raw = {
                "properties": properties,
                "entities": entities
            }

        return raw

    @staticmethod
    def _create_loadbalancer_dict(loadbalancer):
        items = []
        entities = dict()

        properties = {}

        if loadbalancer.name:
            properties['name'] = loadbalancer.name

        # Optional Properties
        if loadbalancer.ip:
            properties['ip'] = loadbalancer.ip
        if loadbalancer.dhcp is not None:
            properties['dhcp'] = str(loadbalancer.dhcp).lower()

        if loadbalancer.balancednics:
            for nic in loadbalancer.balancednics:
                balancednic_properties = {
                    "id": nic
                }
                items.append(balancednic_properties)

            item_entities = {
                "items": items
            }

            balancednics_entities = {
                "balancednics": item_entities
            }

            entities.update(balancednics_entities)

        if not loadbalancer.balancednics:
            raw = {
                "properties": properties,
            }
        else:
            raw = {
                "properties": properties,
                "entities": entities
            }

        return raw

    @staticmethod
    def _create_k8s_dict(cluster_name):
        properties = {
            "name": cluster_name
        }

        raw = {
            "properties": properties,
        }

        return raw

    def _create_nic_dict(self, nic):
        items = []

        properties = {
            "name": nic.name
        }

        if nic.lan:
            properties['lan'] = nic.lan

        # Optional Properties
        if nic.nat:
            properties['nat'] = nic.nat

        if nic.ips:
            properties['ips'] = nic.ips

        if nic.dhcp is not None:
            properties['dhcp'] = nic.dhcp

        if nic.firewall_active is not None:
            properties['firewallActive'] = nic.firewall_active

        if nic.firewall_rules:
            for rule in nic.firewall_rules:
                items.append(self._create_firewallrules_dict(rule))

        rules = {
            "items": items
        }

        entities = {
            "firewallrules": rules
        }

        if not nic.firewall_rules:
            raw = {
                "properties": properties,
            }
        else:
            raw = {
                "properties": properties,
                "entities": entities
            }

        return raw

    @staticmethod
    def _create_firewallrules_dict(rule):
        properties = {}

        if rule.name:
            properties['name'] = rule.name

        if rule.protocol:
            properties['protocol'] = rule.protocol

        if rule.source_mac:
            properties['sourceMac'] = rule.source_mac

        if rule.source_ip:
            properties['sourceIp'] = rule.source_ip

        if rule.target_ip:
            properties['targetIp'] = rule.target_ip

        if rule.port_range_start:
            properties['portRangeStart'] = rule.port_range_start

        if rule.port_range_end:
            properties['portRangeEnd'] = rule.port_range_end

        if rule.icmp_type:
            properties['icmpType'] = rule.icmp_type

        if rule.icmp_code:
            properties['icmpCode'] = rule.icmp_code

        raw = {
            "properties": properties
        }

        return raw

    def _create_server_dict(self, server):
        volume_items = []
        nic_items = []
        entities = dict()

        properties = {
            "name": server.name
        }

        # Omit required attributes, if not provided,
        # to receive a proper error message.
        if server.ram:
            properties['ram'] = server.ram

        if server.cores:
            properties['cores'] = server.cores

        # Optional Properties
        if server.availability_zone:
            properties['availabilityZone'] = server.availability_zone

        if server.boot_cdrom:
            properties['bootCdrom'] = server.boot_cdrom

        if server.boot_volume_id:
            boot_volume = {
                "id": server.boot_volume_id
            }
            properties['bootVolume'] = boot_volume

        if server.cpu_family:
            properties['cpuFamily'] = server.cpu_family

        if server.create_volumes:
            for volume in server.create_volumes:
                volume_items.append(self._create_volume_dict(volume))

            volumes = {
                "items": volume_items
            }

            volume_entities = {
                "volumes": volumes
            }

            entities.update(volume_entities)

        if server.nics:
            for nic in server.nics:
                nic_items.append(self._create_nic_dict(nic))

            nics = {
                "items": nic_items
            }

            nic_entities = {
                "nics": nics
            }
            entities.update(nic_entities)

        # Attach Existing Volume(s)
        if server.attach_volumes:
            for volume in server.attach_volumes:
                volume_properties = {
                    "id": volume
                }
                volume_items.append(volume_properties)

            volumes = {
                "items": volume_items
            }

            volume_entities = {
                "volumes": volumes
            }

            entities.update(volume_entities)

        if not entities:
            raw = {
                "properties": properties,
            }
        else:
            raw = {
                "properties": properties,
                "entities": entities
            }

        return raw

    @staticmethod
    def _create_volume_dict(volume):
        properties = {
            "name": volume.name
        }

        # Omit 'size' attributes, if not provided,
        # to receive a proper error message.
        if volume.size:
            properties['size'] = int(volume.size)

        # Optional Properties
        if volume.availability_zone:
            properties['availabilityZone'] = volume.availability_zone

        if volume.image:
            properties['image'] = volume.image

        if volume.image_alias:
            properties['imageAlias'] = volume.image_alias

        if volume.bus:
            properties['bus'] = volume.bus

        if volume.disk_type:
            properties['type'] = volume.disk_type

        if volume.image is None and volume.image_alias is None:
            properties['licenceType'] = volume.licence_type

        # if volume.licence_type:
        #     properties['licenceType'] = volume.licence_type

        if volume.image_password:
            properties['imagePassword'] = volume.image_password

        if volume.ssh_keys:
            properties['sshKeys'] = volume.ssh_keys

        raw = {
            "properties": properties
        }

        return raw

    @staticmethod
    def _create_group_dict(group):
        properties = {}

        if group.name:
            properties['name'] = group.name

        # Optional Properties
        if group.reserve_ip:
            properties['reserveIp'] = group.reserve_ip

        if group.create_snapshot:
            properties['createSnapshot'] = group.create_snapshot

        if group.create_datacenter:
            properties['createDataCenter'] = \
                group.create_datacenter

        if group.access_activity_log:
            properties['accessActivityLog'] = \
                group.access_activity_log

        raw = {
            "properties": properties
        }

        return raw

    @staticmethod
    def _create_user_dict(user):
        properties = {}

        if user.firstname:
            properties['firstname'] = user.firstname

        if user.lastname:
            properties['lastname'] = user.lastname

        if user.email:
            properties['email'] = user.email

        if user.password:
            properties['password'] = user.password

        # Optional Properties
        if user.administrator:
            properties['administrator'] = user.administrator

        if user.force_sec_auth:
            properties['forceSecAuth'] = user.force_sec_auth

        raw = {
            "properties": properties
        }

        return raw

    @staticmethod
    def _create_privatecrossconnect_dict(privatecrossconnect):
        properties = {}

        if privatecrossconnect.name:
            properties['name'] = privatecrossconnect.name

        if privatecrossconnect.description:
            properties['description'] = privatecrossconnect.description

        raw = {
            "properties": properties
        }

        return raw

    @staticmethod
    def _create_backupunit_dict(backupunit):
        properties = {}

        if backupunit.name:
            properties['name'] = backupunit.name

        if backupunit.password:
            properties['password'] = backupunit.password

        if backupunit.email:
            properties['email'] = backupunit.email

        raw = {
            "properties": properties
        }

        return raw



from ionoscloud.exceptions import ApiException, ApiTimeout, ApiFailedRequest
from ionosenterprise.errors import (
    ICNotAuthorizedError,
    ICNotFoundError,
    ICValidationError,
    ICRateLimitExceededError,
    ICError,
    ICFailedRequest,
    ICTimeoutError
)
import json
import functools
import re
import inspect

class IonosCoreProxy:

    custom_methods = {
        'start_server': 'POST-ACTION',
        'stop_server': 'POST-ACTION',
        'reboot_server': 'POST-ACTION',
        'restore_snapshot': 'POST-ACTION',
        'create_snapshot': 'POST-ACTION-JSON'
    }

    @staticmethod
    def get_default_args(func):
        signature = inspect.signature(func)
        return {
            k: v.default
            for k, v in signature.parameters.items()
            if v.default is not inspect.Parameter.empty
        }

    @staticmethod
    def _request_id(headers):
        # The request URL has currently the format:
        # {host_base}/requests/{request ID}/status
        # Thus search for a UUID.
        match = re.search('/requests/([-A-Fa-f0-9]+)/', headers['location'])
        if match:
            return match.group(1)
        else:
            raise Exception("Failed to extract request ID from response "
                            "header 'location': '{location}'".format(location=headers['location']))

    @staticmethod
    def process_response(f):
        @functools.wraps(f)
        def func(*args, **kwargs):
            try:
                response = f(*args, **kwargs)
                return IonosCoreProxy.handle_response_operations(f, response)
            except Exception as e:
                raise IonosCoreProxy.cast_exception(e)
        return func

    @staticmethod
    def cast_exceptions(f):
        @functools.wraps(f)
        def func(*args, **kwargs):
            try:
                return f(*args, **kwargs)
            except Exception as e:
                if type(e) == ApiException:
                    code = e.status
                    msg = json.loads(e.body)
                    url = e.url
                    if code == 401:
                        raise ICNotAuthorizedError(code, msg, url)
                    elif code == 404:
                        raise ICNotFoundError(code, msg, url)
                    elif code == 422:
                        raise ICValidationError(code, msg, url)
                    elif code == 429:
                        raise ICRateLimitExceededError(code, msg, url)
                    else:
                        raise ICError(code, msg, url)
                elif type(e) == ApiTimeout:
                    raise ICTimeoutError(e.message, e.request_id)
                elif type(e) == ApiFailedRequest:
                    raise ICFailedRequest(e.message, e.request_id)
                raise e
        return func

    @staticmethod
    def _underscore_to_camelcase(f):
        def underscore_to_camelcase(value):
            def camelcase():
                yield str.lower
                while True:
                    yield str.capitalize
            c = camelcase()
            return "".join(next(c)(x) if x else '_' for x in value.split("_"))
        def myprint(d):
            for k in list(d):
                v = d[k]
                del d[k]
                d[underscore_to_camelcase(k)] = v
                if isinstance(v, dict):
                    myprint(v)


        @functools.wraps(f)
        def func(*args, **kwargs):
            x = f(*args, **kwargs)
            myprint(x)
            return x
        return func

    @staticmethod
    def handle_response_operations(func, response):
        return_data = response[0]
        status_code = response[1]
        response_headers = response[2]

        if func.__name__ in IonosCoreProxy.custom_methods and IonosCoreProxy.custom_methods[func.__name__] in ['POST-ACTION-JSON', 'POST-ACTION']:
            if status_code == 202 and func.__name__ in IonosCoreProxy.custom_methods and IonosCoreProxy.custom_methods[func.__name__] == 'POST-ACTION':
                return True

        if 'location' in response_headers:
            if return_data == '':
                return_data = {}
            elif type(return_data) != dict:
                return_data = return_data.to_dict()
            return_data['requestId'] = IonosCoreProxy._request_id(response_headers)

        return return_data



    @staticmethod
    def cast_exception(e):
        if type(e).__name__ == ApiException.__name__:

            code = e.status
            msg = json.loads(e.body)
            url = e.url

            if 'messages' in msg:
                msg = msg['messages']

            if code == 401:
                raise ICNotAuthorizedError(code, msg, url)
            if code == 404:
                raise ICNotFoundError(code, msg, url)
            if code == 422:
                raise ICValidationError(code, msg, url)
            if code == 429:
                raise ICRateLimitExceededError(code, msg, url)
            else:
                raise ICError(code, msg, url)
        raise e